# Python: 3.8.2 (tags/v3.8.2:7b3ab59, Feb 25 2020, 23:03:10) [MSC v.1916 64 bit (AMD64)]
# Library: scipy, version: 1.6.2
# Module: scipy.special.cython_special, version: unspecified
import typing
import builtins as _mod_builtins

__doc__: str
__file__: str
__name__: str
__package__: str
__pyx_capi__: dict
__test__: dict
def _airy_pywrap(x0) -> typing.Any:
    ...

def _airye_pywrap(x0) -> typing.Any:
    ...

def _bench_airy_D_cy() -> typing.Any:
    ...

def _bench_airy_D_py() -> typing.Any:
    ...

def _bench_airy_d_cy() -> typing.Any:
    ...

def _bench_airy_d_py() -> typing.Any:
    ...

def _bench_beta_dd_cy() -> typing.Any:
    ...

def _bench_beta_dd_py() -> typing.Any:
    ...

def _bench_erf_D_cy() -> typing.Any:
    ...

def _bench_erf_D_py() -> typing.Any:
    ...

def _bench_erf_d_cy() -> typing.Any:
    ...

def _bench_erf_d_py() -> typing.Any:
    ...

def _bench_exprel_d_cy() -> typing.Any:
    ...

def _bench_exprel_d_py() -> typing.Any:
    ...

def _bench_gamma_D_cy() -> typing.Any:
    ...

def _bench_gamma_D_py() -> typing.Any:
    ...

def _bench_gamma_d_cy() -> typing.Any:
    ...

def _bench_gamma_d_py() -> typing.Any:
    ...

def _bench_jv_dD_cy() -> typing.Any:
    ...

def _bench_jv_dD_py() -> typing.Any:
    ...

def _bench_jv_dd_cy() -> typing.Any:
    ...

def _bench_jv_dd_py() -> typing.Any:
    ...

def _bench_loggamma_D_cy() -> typing.Any:
    ...

def _bench_loggamma_D_py() -> typing.Any:
    ...

def _bench_logit_d_cy() -> typing.Any:
    ...

def _bench_logit_d_py() -> typing.Any:
    ...

def _bench_psi_D_cy() -> typing.Any:
    ...

def _bench_psi_D_py() -> typing.Any:
    ...

def _bench_psi_d_cy() -> typing.Any:
    ...

def _bench_psi_d_py() -> typing.Any:
    ...

def _ellipj_pywrap() -> typing.Any:
    ...

def _fresnel_pywrap(x0) -> typing.Any:
    ...

def _it2i0k0_pywrap() -> typing.Any:
    ...

def _it2j0y0_pywrap() -> typing.Any:
    ...

def _itairy_pywrap() -> typing.Any:
    ...

def _iti0k0_pywrap() -> typing.Any:
    ...

def _itj0y0_pywrap() -> typing.Any:
    ...

def _kelvin_pywrap() -> typing.Any:
    ...

def _mathieu_cem_pywrap() -> typing.Any:
    ...

def _mathieu_modcem1_pywrap() -> typing.Any:
    ...

def _mathieu_modcem2_pywrap() -> typing.Any:
    ...

def _mathieu_modsem1_pywrap() -> typing.Any:
    ...

def _mathieu_modsem2_pywrap() -> typing.Any:
    ...

def _mathieu_sem_pywrap() -> typing.Any:
    ...

def _modfresnelm_pywrap() -> typing.Any:
    ...

def _modfresnelp_pywrap() -> typing.Any:
    ...

def _obl_ang1_cv_pywrap() -> typing.Any:
    ...

def _obl_ang1_pywrap() -> typing.Any:
    ...

def _obl_rad1_cv_pywrap() -> typing.Any:
    ...

def _obl_rad1_pywrap() -> typing.Any:
    ...

def _obl_rad2_cv_pywrap() -> typing.Any:
    ...

def _obl_rad2_pywrap() -> typing.Any:
    ...

def _pbdv_pywrap() -> typing.Any:
    ...

def _pbvv_pywrap() -> typing.Any:
    ...

def _pbwa_pywrap() -> typing.Any:
    ...

def _pro_ang1_cv_pywrap() -> typing.Any:
    ...

def _pro_ang1_pywrap() -> typing.Any:
    ...

def _pro_rad1_cv_pywrap() -> typing.Any:
    ...

def _pro_rad1_pywrap() -> typing.Any:
    ...

def _pro_rad2_cv_pywrap() -> typing.Any:
    ...

def _pro_rad2_pywrap() -> typing.Any:
    ...

def _shichi_pywrap(x0) -> typing.Any:
    ...

def _sici_pywrap(x0) -> typing.Any:
    ...

def agm() -> typing.Any:
    'See the documentation for scipy.special.agm'
    ...

def bdtr(x0, x1, x2) -> typing.Any:
    'See the documentation for scipy.special.bdtr'
    ...

def bdtrc(x0, x1, x2) -> typing.Any:
    'See the documentation for scipy.special.bdtrc'
    ...

def bdtri(x0, x1, x2) -> typing.Any:
    'See the documentation for scipy.special.bdtri'
    ...

def bdtrik() -> typing.Any:
    'See the documentation for scipy.special.bdtrik'
    ...

def bdtrin() -> typing.Any:
    'See the documentation for scipy.special.bdtrin'
    ...

def bei() -> typing.Any:
    'See the documentation for scipy.special.bei'
    ...

def beip() -> typing.Any:
    'See the documentation for scipy.special.beip'
    ...

def ber() -> typing.Any:
    'See the documentation for scipy.special.ber'
    ...

def berp() -> typing.Any:
    'See the documentation for scipy.special.berp'
    ...

def besselpoly() -> typing.Any:
    'See the documentation for scipy.special.besselpoly'
    ...

def beta() -> typing.Any:
    'See the documentation for scipy.special.beta'
    ...

def betainc() -> typing.Any:
    'See the documentation for scipy.special.betainc'
    ...

def betaincinv() -> typing.Any:
    'See the documentation for scipy.special.betaincinv'
    ...

def betaln() -> typing.Any:
    'See the documentation for scipy.special.betaln'
    ...

def binom() -> typing.Any:
    'See the documentation for scipy.special.binom'
    ...

def boxcox() -> typing.Any:
    'See the documentation for scipy.special.boxcox'
    ...

def boxcox1p() -> typing.Any:
    'See the documentation for scipy.special.boxcox1p'
    ...

def btdtr() -> typing.Any:
    'See the documentation for scipy.special.btdtr'
    ...

def btdtri() -> typing.Any:
    'See the documentation for scipy.special.btdtri'
    ...

def btdtria() -> typing.Any:
    'See the documentation for scipy.special.btdtria'
    ...

def btdtrib() -> typing.Any:
    'See the documentation for scipy.special.btdtrib'
    ...

def cbrt() -> typing.Any:
    'See the documentation for scipy.special.cbrt'
    ...

def chdtr() -> typing.Any:
    'See the documentation for scipy.special.chdtr'
    ...

def chdtrc() -> typing.Any:
    'See the documentation for scipy.special.chdtrc'
    ...

def chdtri() -> typing.Any:
    'See the documentation for scipy.special.chdtri'
    ...

def chdtriv() -> typing.Any:
    'See the documentation for scipy.special.chdtriv'
    ...

def chndtr() -> typing.Any:
    'See the documentation for scipy.special.chndtr'
    ...

def chndtridf() -> typing.Any:
    'See the documentation for scipy.special.chndtridf'
    ...

def chndtrinc() -> typing.Any:
    'See the documentation for scipy.special.chndtrinc'
    ...

def chndtrix() -> typing.Any:
    'See the documentation for scipy.special.chndtrix'
    ...

def cosdg() -> typing.Any:
    'See the documentation for scipy.special.cosdg'
    ...

def cosm1() -> typing.Any:
    'See the documentation for scipy.special.cosm1'
    ...

def cotdg() -> typing.Any:
    'See the documentation for scipy.special.cotdg'
    ...

def dawsn(x0) -> typing.Any:
    'See the documentation for scipy.special.dawsn'
    ...

def ellipe() -> typing.Any:
    'See the documentation for scipy.special.ellipe'
    ...

def ellipeinc() -> typing.Any:
    'See the documentation for scipy.special.ellipeinc'
    ...

def ellipk() -> typing.Any:
    'See the documentation for scipy.special.ellipk'
    ...

def ellipkinc() -> typing.Any:
    'See the documentation for scipy.special.ellipkinc'
    ...

def ellipkm1() -> typing.Any:
    'See the documentation for scipy.special.ellipkm1'
    ...

def entr() -> typing.Any:
    'See the documentation for scipy.special.entr'
    ...

def erf(x0) -> typing.Any:
    'See the documentation for scipy.special.erf'
    ...

def erfc(x0) -> typing.Any:
    'See the documentation for scipy.special.erfc'
    ...

def erfcinv() -> typing.Any:
    'See the documentation for scipy.special.erfcinv'
    ...

def erfcx(x0) -> typing.Any:
    'See the documentation for scipy.special.erfcx'
    ...

def erfi(x0) -> typing.Any:
    'See the documentation for scipy.special.erfi'
    ...

def erfinv() -> typing.Any:
    'See the documentation for scipy.special.erfinv'
    ...

def eval_chebyc(x0, x1) -> typing.Any:
    'See the documentation for scipy.special.eval_chebyc'
    ...

def eval_chebys(x0, x1) -> typing.Any:
    'See the documentation for scipy.special.eval_chebys'
    ...

def eval_chebyt(x0, x1) -> typing.Any:
    'See the documentation for scipy.special.eval_chebyt'
    ...

def eval_chebyu(x0, x1) -> typing.Any:
    'See the documentation for scipy.special.eval_chebyu'
    ...

def eval_gegenbauer(x0, x1, x2) -> typing.Any:
    'See the documentation for scipy.special.eval_gegenbauer'
    ...

def eval_genlaguerre(x0, x1, x2) -> typing.Any:
    'See the documentation for scipy.special.eval_genlaguerre'
    ...

def eval_hermite() -> typing.Any:
    'See the documentation for scipy.special.eval_hermite'
    ...

def eval_hermitenorm() -> typing.Any:
    'See the documentation for scipy.special.eval_hermitenorm'
    ...

def eval_jacobi(x0, x1, x2, x3) -> typing.Any:
    'See the documentation for scipy.special.eval_jacobi'
    ...

def eval_laguerre(x0, x1) -> typing.Any:
    'See the documentation for scipy.special.eval_laguerre'
    ...

def eval_legendre(x0, x1) -> typing.Any:
    'See the documentation for scipy.special.eval_legendre'
    ...

def eval_sh_chebyt(x0, x1) -> typing.Any:
    'See the documentation for scipy.special.eval_sh_chebyt'
    ...

def eval_sh_chebyu(x0, x1) -> typing.Any:
    'See the documentation for scipy.special.eval_sh_chebyu'
    ...

def eval_sh_jacobi(x0, x1, x2, x3) -> typing.Any:
    'See the documentation for scipy.special.eval_sh_jacobi'
    ...

def eval_sh_legendre(x0, x1) -> typing.Any:
    'See the documentation for scipy.special.eval_sh_legendre'
    ...

def exp1(x0) -> typing.Any:
    'See the documentation for scipy.special.exp1'
    ...

def exp10() -> typing.Any:
    'See the documentation for scipy.special.exp10'
    ...

def exp2() -> typing.Any:
    'See the documentation for scipy.special.exp2'
    ...

def expi(x0) -> typing.Any:
    'See the documentation for scipy.special.expi'
    ...

def expit(x0) -> typing.Any:
    'See the documentation for scipy.special.expit'
    ...

def expm1(x0) -> typing.Any:
    'See the documentation for scipy.special.expm1'
    ...

def expn(x0, x1) -> typing.Any:
    'See the documentation for scipy.special.expn'
    ...

def exprel() -> typing.Any:
    'See the documentation for scipy.special.exprel'
    ...

def fdtr() -> typing.Any:
    'See the documentation for scipy.special.fdtr'
    ...

def fdtrc() -> typing.Any:
    'See the documentation for scipy.special.fdtrc'
    ...

def fdtri() -> typing.Any:
    'See the documentation for scipy.special.fdtri'
    ...

def fdtridfd() -> typing.Any:
    'See the documentation for scipy.special.fdtridfd'
    ...

def gamma(x0) -> typing.Any:
    'See the documentation for scipy.special.gamma'
    ...

def gammainc() -> typing.Any:
    'See the documentation for scipy.special.gammainc'
    ...

def gammaincc() -> typing.Any:
    'See the documentation for scipy.special.gammaincc'
    ...

def gammainccinv() -> typing.Any:
    'See the documentation for scipy.special.gammainccinv'
    ...

def gammaincinv() -> typing.Any:
    'See the documentation for scipy.special.gammaincinv'
    ...

def gammaln() -> typing.Any:
    'See the documentation for scipy.special.gammaln'
    ...

def gammasgn() -> typing.Any:
    'See the documentation for scipy.special.gammasgn'
    ...

def gdtr() -> typing.Any:
    'See the documentation for scipy.special.gdtr'
    ...

def gdtrc() -> typing.Any:
    'See the documentation for scipy.special.gdtrc'
    ...

def gdtria() -> typing.Any:
    'See the documentation for scipy.special.gdtria'
    ...

def gdtrib() -> typing.Any:
    'See the documentation for scipy.special.gdtrib'
    ...

def gdtrix() -> typing.Any:
    'See the documentation for scipy.special.gdtrix'
    ...

def hankel1() -> typing.Any:
    'See the documentation for scipy.special.hankel1'
    ...

def hankel1e() -> typing.Any:
    'See the documentation for scipy.special.hankel1e'
    ...

def hankel2() -> typing.Any:
    'See the documentation for scipy.special.hankel2'
    ...

def hankel2e() -> typing.Any:
    'See the documentation for scipy.special.hankel2e'
    ...

def huber() -> typing.Any:
    'See the documentation for scipy.special.huber'
    ...

def hyp0f1(x0, x1) -> typing.Any:
    'See the documentation for scipy.special.hyp0f1'
    ...

def hyp1f1(x0, x1, x2) -> typing.Any:
    'See the documentation for scipy.special.hyp1f1'
    ...

def hyp2f1(x0, x1, x2, x3) -> typing.Any:
    'See the documentation for scipy.special.hyp2f1'
    ...

def hyperu() -> typing.Any:
    'See the documentation for scipy.special.hyperu'
    ...

def i0() -> typing.Any:
    'See the documentation for scipy.special.i0'
    ...

def i0e() -> typing.Any:
    'See the documentation for scipy.special.i0e'
    ...

def i1() -> typing.Any:
    'See the documentation for scipy.special.i1'
    ...

def i1e() -> typing.Any:
    'See the documentation for scipy.special.i1e'
    ...

def inv_boxcox() -> typing.Any:
    'See the documentation for scipy.special.inv_boxcox'
    ...

def inv_boxcox1p() -> typing.Any:
    'See the documentation for scipy.special.inv_boxcox1p'
    ...

def it2struve0() -> typing.Any:
    'See the documentation for scipy.special.it2struve0'
    ...

def itmodstruve0() -> typing.Any:
    'See the documentation for scipy.special.itmodstruve0'
    ...

def itstruve0() -> typing.Any:
    'See the documentation for scipy.special.itstruve0'
    ...

def iv(x0, x1) -> typing.Any:
    'See the documentation for scipy.special.iv'
    ...

def ive(x0, x1) -> typing.Any:
    'See the documentation for scipy.special.ive'
    ...

def j0() -> typing.Any:
    'See the documentation for scipy.special.j0'
    ...

def j1() -> typing.Any:
    'See the documentation for scipy.special.j1'
    ...

def jv(x0, x1) -> typing.Any:
    'See the documentation for scipy.special.jv'
    ...

def jve(x0, x1) -> typing.Any:
    'See the documentation for scipy.special.jve'
    ...

def k0() -> typing.Any:
    'See the documentation for scipy.special.k0'
    ...

def k0e() -> typing.Any:
    'See the documentation for scipy.special.k0e'
    ...

def k1() -> typing.Any:
    'See the documentation for scipy.special.k1'
    ...

def k1e() -> typing.Any:
    'See the documentation for scipy.special.k1e'
    ...

def kei() -> typing.Any:
    'See the documentation for scipy.special.kei'
    ...

def keip() -> typing.Any:
    'See the documentation for scipy.special.keip'
    ...

def ker() -> typing.Any:
    'See the documentation for scipy.special.ker'
    ...

def kerp() -> typing.Any:
    'See the documentation for scipy.special.kerp'
    ...

def kl_div() -> typing.Any:
    'See the documentation for scipy.special.kl_div'
    ...

def kn(x0, x1) -> typing.Any:
    'See the documentation for scipy.special.kn'
    ...

def kolmogi() -> typing.Any:
    'See the documentation for scipy.special.kolmogi'
    ...

def kolmogorov() -> typing.Any:
    'See the documentation for scipy.special.kolmogorov'
    ...

def kv(x0, x1) -> typing.Any:
    'See the documentation for scipy.special.kv'
    ...

def kve(x0, x1) -> typing.Any:
    'See the documentation for scipy.special.kve'
    ...

def log1p(x0) -> typing.Any:
    'See the documentation for scipy.special.log1p'
    ...

def log_ndtr(x0) -> typing.Any:
    'See the documentation for scipy.special.log_ndtr'
    ...

def loggamma(x0) -> typing.Any:
    'See the documentation for scipy.special.loggamma'
    ...

def logit(x0) -> typing.Any:
    'See the documentation for scipy.special.logit'
    ...

def lpmv() -> typing.Any:
    'See the documentation for scipy.special.lpmv'
    ...

def mathieu_a() -> typing.Any:
    'See the documentation for scipy.special.mathieu_a'
    ...

def mathieu_b() -> typing.Any:
    'See the documentation for scipy.special.mathieu_b'
    ...

def modstruve() -> typing.Any:
    'See the documentation for scipy.special.modstruve'
    ...

def nbdtr(x0, x1, x2) -> typing.Any:
    'See the documentation for scipy.special.nbdtr'
    ...

def nbdtrc(x0, x1, x2) -> typing.Any:
    'See the documentation for scipy.special.nbdtrc'
    ...

def nbdtri(x0, x1, x2) -> typing.Any:
    'See the documentation for scipy.special.nbdtri'
    ...

def nbdtrik() -> typing.Any:
    'See the documentation for scipy.special.nbdtrik'
    ...

def nbdtrin() -> typing.Any:
    'See the documentation for scipy.special.nbdtrin'
    ...

def ncfdtr() -> typing.Any:
    'See the documentation for scipy.special.ncfdtr'
    ...

def ncfdtri() -> typing.Any:
    'See the documentation for scipy.special.ncfdtri'
    ...

def ncfdtridfd() -> typing.Any:
    'See the documentation for scipy.special.ncfdtridfd'
    ...

def ncfdtridfn() -> typing.Any:
    'See the documentation for scipy.special.ncfdtridfn'
    ...

def ncfdtrinc() -> typing.Any:
    'See the documentation for scipy.special.ncfdtrinc'
    ...

def nctdtr() -> typing.Any:
    'See the documentation for scipy.special.nctdtr'
    ...

def nctdtridf() -> typing.Any:
    'See the documentation for scipy.special.nctdtridf'
    ...

def nctdtrinc() -> typing.Any:
    'See the documentation for scipy.special.nctdtrinc'
    ...

def nctdtrit() -> typing.Any:
    'See the documentation for scipy.special.nctdtrit'
    ...

def ndtr(x0) -> typing.Any:
    'See the documentation for scipy.special.ndtr'
    ...

def ndtri() -> typing.Any:
    'See the documentation for scipy.special.ndtri'
    ...

def nrdtrimn() -> typing.Any:
    'See the documentation for scipy.special.nrdtrimn'
    ...

def nrdtrisd() -> typing.Any:
    'See the documentation for scipy.special.nrdtrisd'
    ...

def obl_cv() -> typing.Any:
    'See the documentation for scipy.special.obl_cv'
    ...

def owens_t() -> typing.Any:
    'See the documentation for scipy.special.owens_t'
    ...

def pdtr() -> typing.Any:
    'See the documentation for scipy.special.pdtr'
    ...

def pdtrc() -> typing.Any:
    'See the documentation for scipy.special.pdtrc'
    ...

def pdtri(x0, x1) -> typing.Any:
    'See the documentation for scipy.special.pdtri'
    ...

def pdtrik() -> typing.Any:
    'See the documentation for scipy.special.pdtrik'
    ...

def poch() -> typing.Any:
    'See the documentation for scipy.special.poch'
    ...

def pro_cv() -> typing.Any:
    'See the documentation for scipy.special.pro_cv'
    ...

def pseudo_huber() -> typing.Any:
    'See the documentation for scipy.special.pseudo_huber'
    ...

def psi(x0) -> typing.Any:
    'See the documentation for scipy.special.psi'
    ...

def radian() -> typing.Any:
    'See the documentation for scipy.special.radian'
    ...

def rel_entr() -> typing.Any:
    'See the documentation for scipy.special.rel_entr'
    ...

def rgamma(x0) -> typing.Any:
    'See the documentation for scipy.special.rgamma'
    ...

def round() -> typing.Any:
    'See the documentation for scipy.special.round'
    ...

def sindg() -> typing.Any:
    'See the documentation for scipy.special.sindg'
    ...

def smirnov(x0, x1) -> typing.Any:
    'See the documentation for scipy.special.smirnov'
    ...

def smirnovi(x0, x1) -> typing.Any:
    'See the documentation for scipy.special.smirnovi'
    ...

def spence(x0) -> typing.Any:
    'See the documentation for scipy.special.spence'
    ...

def sph_harm(x0, x1, x2, x3) -> typing.Any:
    'See the documentation for scipy.special.sph_harm'
    ...

def spherical_in() -> typing.Any:
    'See the documentation for scipy.special.spherical_in'
    ...

def spherical_jn() -> typing.Any:
    'See the documentation for scipy.special.spherical_jn'
    ...

def spherical_kn() -> typing.Any:
    'See the documentation for scipy.special.spherical_kn'
    ...

def spherical_yn() -> typing.Any:
    'See the documentation for scipy.special.spherical_yn'
    ...

def stdtr() -> typing.Any:
    'See the documentation for scipy.special.stdtr'
    ...

def stdtridf() -> typing.Any:
    'See the documentation for scipy.special.stdtridf'
    ...

def stdtrit() -> typing.Any:
    'See the documentation for scipy.special.stdtrit'
    ...

def struve() -> typing.Any:
    'See the documentation for scipy.special.struve'
    ...

def tandg() -> typing.Any:
    'See the documentation for scipy.special.tandg'
    ...

def tklmbda() -> typing.Any:
    'See the documentation for scipy.special.tklmbda'
    ...

def voigt_profile() -> typing.Any:
    'See the documentation for scipy.special.voigt_profile'
    ...

def wofz() -> typing.Any:
    'See the documentation for scipy.special.wofz'
    ...

def wrightomega(x0) -> typing.Any:
    'See the documentation for scipy.special.wrightomega'
    ...

def xlog1py(x0, x1) -> typing.Any:
    'See the documentation for scipy.special.xlog1py'
    ...

def xlogy(x0, x1) -> typing.Any:
    'See the documentation for scipy.special.xlogy'
    ...

def y0() -> typing.Any:
    'See the documentation for scipy.special.y0'
    ...

def y1() -> typing.Any:
    'See the documentation for scipy.special.y1'
    ...

def yn(x0, x1) -> typing.Any:
    'See the documentation for scipy.special.yn'
    ...

def yv(x0, x1) -> typing.Any:
    'See the documentation for scipy.special.yv'
    ...

def yve(x0, x1) -> typing.Any:
    'See the documentation for scipy.special.yve'
    ...

def zetac() -> typing.Any:
    'See the documentation for scipy.special.zetac'
    ...

def __getattr__(name) -> typing.Any:
    ...




"""
This type stub file was generated by pyright.

Created on 5/30/2023 against 8.13.2 of IPython.

"""

import sys
from .core.getipython import get_ipython
from .core import release
from .core.application import Application
from .terminal.embed import embed
from .core.interactiveshell import InteractiveShell
from .utils.sysinfo import sys_info
from .utils.frame import extract_module_locals

"""
IPython: tools for interactive and parallel computing in Python.

https://ipython.org
"""
if sys.version_info < (3, 9):
    ...
__all__ = ["start_ipython", "embed", "start_kernel", "embed_kernel"]
__author__ = ...
__license__ = ...
__version__ = ...
version_info = ...
__patched_cves__ = ...
def embed_kernel(module=..., local_ns=..., **kwargs): # -> None:
    """Embed and start an IPython kernel in a given scope.

    If you don't want the kernel to initialize the namespace
    from the scope of the surrounding function,
    and/or you want to load full IPython configuration,
    you probably want `IPython.start_kernel()` instead.

    Parameters
    ----------
    module : types.ModuleType, optional
        The module to load into IPython globals (default: caller)
    local_ns : dict, optional
        The namespace to load into IPython user namespace (default: caller)
    **kwargs : various, optional
        Further keyword args are relayed to the IPKernelApp constructor,
        such as `config`, a traitlets :class:`Config` object (see :ref:`configure_start_ipython`),
        allowing configuration of the kernel (see :ref:`kernel_options`).  Will only have an effect
        on the first embed_kernel call for a given process.
    """
    ...

def start_ipython(argv=..., **kwargs): # -> None:
    """Launch a normal IPython instance (as opposed to embedded)

    `IPython.embed()` puts a shell in a particular calling scope,
    such as a function or method for debugging purposes,
    which is often not desirable.

    `start_ipython()` does full, regular IPython initialization,
    including loading startup files, configuration, etc.
    much of which is skipped by `embed()`.

    This is a public API method, and will survive implementation changes.

    Parameters
    ----------
    argv : list or None, optional
        If unspecified or None, IPython will parse command-line options from sys.argv.
        To prevent any command-line parsing, pass an empty list: `argv=[]`.
    user_ns : dict, optional
        specify this dictionary to initialize the IPython user namespace with particular values.
    **kwargs : various, optional
        Any other kwargs will be passed to the Application constructor,
        such as `config`, a traitlets :class:`Config` object (see :ref:`configure_start_ipython`),
        allowing configuration of the instance (see :ref:`terminal_options`).
    """
    ...

def start_kernel(argv=..., **kwargs): # -> None:
    """Launch a normal IPython kernel instance (as opposed to embedded)

    `IPython.embed_kernel()` puts a shell in a particular calling scope,
    such as a function or method for debugging purposes,
    which is often not desirable.

    `start_kernel()` does full, regular IPython initialization,
    including loading startup files, configuration, etc.
    much of which is skipped by `embed_kernel()`.

    Parameters
    ----------
    argv : list or None, optional
        If unspecified or None, IPython will parse command-line options from sys.argv.
        To prevent any command-line parsing, pass an empty list: `argv=[]`.
    user_ns : dict, optional
        specify this dictionary to initialize the IPython user namespace with particular values.
    **kwargs : various, optional
        Any other kwargs will be passed to the Application constructor,
        such as `config`, a traitlets :class:`Config` object (see :ref:`configure_start_ipython`),
        allowing configuration of the kernel (see :ref:`kernel_options`).
    """
    ...




from typing import Any, Type
from collections import OrderedDict as OrderedDict
from .._config import get_config as get_config
from ..base import BaseEstimator as BaseEstimator
from . import is_scalar_nan as is_scalar_nan

# Copyright (c) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010,
# 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018 Python Software Foundation;
# All Rights Reserved

# Authors: Fred L. Drake, Jr. <fdrake@acm.org> (built-in CPython pprint module)
#          Nicolas Hug (scikit-learn specific changes)

# License: PSF License version 2 (see below)

# PYTHON SOFTWARE FOUNDATION LICENSE VERSION 2
# --------------------------------------------

# 1. This LICENSE AGREEMENT is between the Python Software Foundation ("PSF"),
# and the Individual or Organization ("Licensee") accessing and otherwise
# using this software ("Python") in source or binary form and its associated
# documentation.

# 2. Subject to the terms and conditions of this License Agreement, PSF hereby
# grants Licensee a nonexclusive, royalty-free, world-wide license to
# reproduce, analyze, test, perform and/or display publicly, prepare
# derivative works, distribute, and otherwise use Python alone or in any
# derivative version, provided, however, that PSF's License Agreement and
# PSF's notice of copyright, i.e., "Copyright (c) 2001, 2002, 2003, 2004,
# 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
# 2017, 2018 Python Software Foundation; All Rights Reserved" are retained in
# Python alone or in any derivative version prepared by Licensee.

# 3. In the event Licensee prepares a derivative work that is based on or
# incorporates Python or any part thereof, and wants to make the derivative
# work available to others as provided herein, then Licensee hereby agrees to
# include in any such work a brief summary of the changes made to Python.

# 4. PSF is making Python available to Licensee on an "AS IS" basis. PSF MAKES
# NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR IMPLIED. BY WAY OF EXAMPLE, BUT
# NOT LIMITATION, PSF MAKES NO AND DISCLAIMS ANY REPRESENTATION OR WARRANTY OF
# MERCHANTABILITY OR FITNESS FOR ANY PARTICULAR PURPOSE OR THAT THE USE OF
# PYTHON WILL NOT INFRINGE ANY THIRD PARTY RIGHTS.

# 5. PSF SHALL NOT BE LIABLE TO LICENSEE OR ANY OTHER USERS OF PYTHON FOR ANY
# INCIDENTAL, SPECIAL, OR CONSEQUENTIAL DAMAGES OR LOSS AS A RESULT OF
# MODIFYING, DISTRIBUTING, OR OTHERWISE USING PYTHON, OR ANY DERIVATIVE
# THEREOF, EVEN IF ADVISED OF THE POSSIBILITY THEREOF.

# 6. This License Agreement will automatically terminate upon a material
# breach of its terms and conditions.

# 7. Nothing in this License Agreement shall be deemed to create any
# relationship of agency, partnership, or joint venture between PSF and
# Licensee. This License Agreement does not grant permission to use PSF
# trademarks or trade name in a trademark sense to endorse or promote products
# or services of Licensee, or any third party.

# 8. By copying, installing or otherwise using Python, Licensee agrees to be
# bound by the terms and conditions of this License Agreement.


# Brief summary of changes to original code:
# - "compact" parameter is supported for dicts, not just lists or tuples
# - estimators have a custom handler, they're not just treated as objects
# - long sequences (lists, tuples, dict items) with more than N elements are
#   shortened using ellipsis (', ...') at the end.

import inspect
import pprint


class KeyValTuple(tuple):
    def __repr__(self) -> str:
        ...


class KeyValTupleParam(KeyValTuple):
    ...


class _EstimatorPrettyPrinter(pprint.PrettyPrinter):
    def __init__(
        self,
        indent: int = 1,
        width: int = 80,
        depth=None,
        stream=None,
        *,
        compact: bool = False,
        indent_at_name: bool = True,
        n_max_elements_to_show=None,
    ) -> None:
        ...

    def format(
        self, object, context: dict[Any, Any] | dict[int, int], maxlevels, level: int
    ) -> tuple[str, bool, bool]:
        ...

    # Note: need to copy _dispatch to prevent instances of the builtin
    # PrettyPrinter class to call methods of _EstimatorPrettyPrinter (see issue
    # 12906)
    # mypy error: "Type[PrettyPrinter]" has no attribute "_dispatch"
    _dispatch = ...  # type: ignore



# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

from __future__ import division

import numpy as np

from ...util import transforms
from ...util.quaternion import Quaternion
from ...visuals.transforms import MatrixTransform
from .perspective import Base3DRotationCamera

class ArcballCamera(Base3DRotationCamera):

    _state_props = ...

    def __init__(self, fov: float = 45.0, distance: None | float = None, translate_speed: float = 1.0, **kwargs): ...
    def _update_rotation(self, event): ...
    def _get_rotation_tr(self): ...
    def _dist_to_trans(self, dist): ...
    def _get_dim_vectors(self): ...

def _arcball(xy, wh): ...



from numpy.typing import ArrayLike
from ...classes.graph import Graph

__all__ = ["generate_edgelist", "write_edgelist", "parse_edgelist", "read_edgelist"]

from ...utils import not_implemented_for, open_file

def write_edgelist(
    G: Graph,
    path,
    comments: str = "#",
    delimiter: str = " ",
    data: bool | ArrayLike = True,
    encoding: str = "utf-8",
): ...
def generate_edgelist(G: Graph, delimiter: str = " ", data=True) -> str: ...
def parse_edgelist(
    lines,
    comments: str = "#",
    delimiter: str | None = None,
    create_using=None,
    nodetype=None,
    data=True,
): ...
def read_edgelist(
    path,
    comments: str = "#",
    delimiter: str | None = None,
    create_using=None,
    nodetype=None,
    data=True,
    edgetype=None,
    encoding: str = "utf-8",
): ...



from typing import Mapping
import itertools
from collections import defaultdict
from collections.abc import Mapping
from functools import cached_property

from ...classes.graph import Graph
from . import local_node_connectivity
from ...exception import NetworkXError
from ...utils import not_implemented_for

__all__ = ["k_components"]

def k_components(G: Graph, min_density: float = 0.95) -> Mapping: ...

class _AntiGraph(Graph):

    all_edge_dict: dict = ...

    def single_edge_dict(self): ...

    edge_attr_dict_factory = single_edge_dict  # type: ignore

    def __getitem__(self, n) -> Mapping: ...
    def neighbors(self, n): ...

    ...
    ...

    @cached_property
    def adj(self): ...
    def subgraph(self, nodes): ...

    ...

    @cached_property
    def degree(self): ...
    def adjacency(self): ...



from vispy.util.event import Event

# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.
# -----------------------------------------------------------------------------

from .widget import Widget
from ...visuals import TextVisual

class Label(Widget):
    def __init__(self, text: str, rotation: float = 0.0, **kwargs): ...
    def on_resize(self, event: Event): ...
    def _set_pos(self): ...
    @property
    def text(self): ...
    @text.setter
    def text(self, t): ...



from ..classes.graph import Graph

import itertools
import math
from collections import defaultdict

from ..classes.graph import Graph
from ..utils import py_random_state

from .classic import complete_graph, empty_graph, path_graph, star_graph
from .degree_seq import degree_sequence_tree

__all__ = [
    "fast_gnp_random_graph",
    "gnp_random_graph",
    "dense_gnm_random_graph",
    "gnm_random_graph",
    "erdos_renyi_graph",
    "binomial_graph",
    "newman_watts_strogatz_graph",
    "watts_strogatz_graph",
    "connected_watts_strogatz_graph",
    "random_regular_graph",
    "barabasi_albert_graph",
    "dual_barabasi_albert_graph",
    "extended_barabasi_albert_graph",
    "powerlaw_cluster_graph",
    "random_lobster",
    "random_shell_graph",
    "random_powerlaw_tree",
    "random_powerlaw_tree_sequence",
    "random_kernel_graph",
]

@py_random_state(2)
def fast_gnp_random_graph(n: int, p: float, seed=None, directed=False): ...
@py_random_state(2)
def gnp_random_graph(n: int, p: float, seed=None, directed=False): ...

# add some aliases to common names
binomial_graph = ...
erdos_renyi_graph = ...

@py_random_state(2)
def dense_gnm_random_graph(n: int, m: int, seed=None): ...
@py_random_state(2)
def gnm_random_graph(n: int, m: int, seed=None, directed=False): ...
@py_random_state(3)
def newman_watts_strogatz_graph(n: int, k: int, p: float, seed=None): ...
@py_random_state(3)
def watts_strogatz_graph(n: int, k: int, p: float, seed=None): ...
@py_random_state(4)
def connected_watts_strogatz_graph(
    n: int, k: int, p: float, tries: int = 100, seed=None
): ...
@py_random_state(2)
def random_regular_graph(d: int, n, seed=None): ...
@py_random_state(2)
def barabasi_albert_graph(
    n: int, m: int, seed=None, initial_graph: Graph | None = None
) -> Graph: ...
@py_random_state(4)
def dual_barabasi_albert_graph(
    n: int, m1: int, m2: int, p: float, seed=None, initial_graph: Graph | None = None
) -> Graph: ...
@py_random_state(4)
def extended_barabasi_albert_graph(
    n: int, m: int, p: float, q: float, seed=None
) -> Graph: ...
@py_random_state(3)
def powerlaw_cluster_graph(n: int, m: int, p: float, seed=None): ...
@py_random_state(3)
def random_lobster(n: int, p1: float, p2: float, seed=None): ...
@py_random_state(1)
def random_shell_graph(constructor, seed=None): ...
@py_random_state(2)
def random_powerlaw_tree(n: int, gamma: float = 3, seed=None, tries: int = 100): ...
@py_random_state(2)
def random_powerlaw_tree_sequence(
    n: int, gamma: float = 3, seed=None, tries: int = 100
): ...
@py_random_state(3)
def random_kernel_graph(n: int, kernel_integral, kernel_root=None, seed=None): ...



from vispy.scene.node import Node
from vispy.util.event import Event

# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

import numpy as np

from .widget import Widget
from ..subscene import SubScene
from ..cameras import make_camera, BaseCamera
from ...visuals.filters import Clipper

class ViewBox(Widget):
    def __init__(self, camera: None | str | BaseCamera = None, **kwargs): ...
    @property
    def camera(self): ...
    @camera.setter
    def camera(self, cam): ...
    def is_in_scene(self, node: Node): ...
    def get_scene_bounds(self, dim: None | int = None) -> list | tuple: ...
    @property
    def scene(self): ...
    def add(self, node: Node): ...
    def on_resize(self, event: Event): ...
    def _update_scene_clipper(self, event=None): ...



import json
import warnings

from ...classes.graph import Graph
from ...utils.decorators import not_implemented_for

__all__ = ["jit_graph", "jit_data"]

def jit_graph(data, create_using=None): ...
def jit_data(G: Graph, indent=None, default=None): ...



# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.
from collections import OrderedDict
from weakref import WeakKeyDictionary

from .compiler import Compiler

class ShaderObject(object):
    @classmethod
    def create(self, obj, ref=None): ...
    def __init__(self): ...
    @property
    def name(self): ...
    @property
    def version_pragma(self): ...
    def definition(self, obj_names, version, shader): ...
    def expression(self, obj_names): ...
    def dependencies(self, sort=False): ...
    def static_names(self): ...
    def _add_dep(self, dep): ...
    def _remove_dep(self, dep): ...
    def _dep_changed(self, dep, code_changed=False, value_changed=False): ...
    def changed(self, code_changed=False, value_changed=False): ...
    def compile(self): ...
    def __repr__(self): ...

from .variable import Variable  # noqa
from .expression import TextExpression  # noqa



from typing import Mapping, Callable
import os
import tempfile

from ..classes.graph import Graph

__all__ = [
    "from_agraph",
    "to_agraph",
    "write_dot",
    "read_dot",
    "graphviz_layout",
    "pygraphviz_layout",
    "view_pygraphviz",
]

def from_agraph(A, create_using=None): ...
def to_agraph(N): ...
def write_dot(G: Graph, path): ...
def read_dot(path): ...
def graphviz_layout(
    G: Graph, prog: str = "neato", root: str | None = None, args: str = ""
): ...
def pygraphviz_layout(
    G: Graph, prog: str = "neato", root: str | None = None, args: str = ""
) -> Mapping: ...
def view_pygraphviz(
    G: Graph,
    edgelabel: str | Callable | None = None,
    prog: str = "dot",
    args: str = "",
    suffix: str = "",
    path: str | None = None,
    show: bool = True,
): ...
def display_pygraphviz(
    graph, path, format: str | None = None, prog: str | None = None, args: str = ""
): ...



import math

from ..classes.graph import Graph
from ..utils import py_random_state

__all__ = ["double_edge_swap", "connected_double_edge_swap"]

@py_random_state(3)
def double_edge_swap(G: Graph, nswap=1, max_tries=100, seed=None): ...
@py_random_state(3)
def connected_double_edge_swap(
    G: Graph, nswap=1, _window_threshold=3, seed=None
) -> int: ...



from typing import Callable, Literal
from ...utils.validation import (
    check_consistent_length as check_consistent_length,
    check_array as check_array,
)
from numpy import ndarray
from ..._typing import Float
from scipy.optimize import linear_sum_assignment as linear_sum_assignment
import numpy as np

__all__ = ["consensus_score"]


def consensus_score(
    a: tuple[ndarray, ndarray] | tuple[int, int],
    b: tuple[ndarray, ndarray] | tuple[int, int],
    *,
    similarity: Literal["jaccard", "jaccard"] | Callable = "jaccard"
) -> Float:
    ...



from numpy.typing import NDArray

# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.
# This module was taken from visvis

import numpy as np
import time
from gzip import GzipFile
from os import path as op

from ..geometry import _calculate_normals
from ..util import logger

class WavefrontReader(object):
    def __init__(self, f): ...
    @classmethod
    def read(cls, fname: str): ...
    def readLine(self): ...
    def readTuple(self, line, n=3): ...
    def readFace(self, line): ...
    def _absint(self, i, ref): ...
    def _calculate_normals(self): ...
    def finish(self): ...

class WavefrontWriter(object):
    def __init__(self, f): ...
    @classmethod
    def write(
        cls,
        fname: str,
        vertices: NDArray,
        faces: NDArray,
        normals,
        texcoords: NDArray,
        name: str = "",
        reshape_faces: bool = True,
    ): ...
    def writeLine(self, text): ...
    def writeTuple(self, val, what): ...
    def writeFace(self, val, what="f"): ...
    def writeMesh(self, vertices, faces, normals, values, name="", reshape_faces=True): ...



from typing import Any, ClassVar, TypeVar
from ..utils.sparsefuncs import (
    mean_variance_axis as mean_variance_axis,
    min_max_axis as min_max_axis,
)
from ._base import SelectorMixin
from numpy import ndarray
from ..utils._param_validation import Interval as Interval
from numbers import Real as Real
from ..base import BaseEstimator
from .._typing import Float, MatrixLike
from ..utils.validation import check_is_fitted as check_is_fitted

VarianceThreshold_Self = TypeVar("VarianceThreshold_Self", bound="VarianceThreshold")

# Author: Lars Buitinck
# License: 3-clause BSD

import numpy as np


class VarianceThreshold(SelectorMixin, BaseEstimator):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    variances_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(self, threshold: Float = 0.0) -> None:
        ...

    def fit(
        self: VarianceThreshold_Self, X: MatrixLike, y: Any = None
    ) -> VarianceThreshold_Self:
        ...



from ._base import MissingIndicator as MissingIndicator, SimpleImputer as SimpleImputer
from ._knn import KNNImputer as KNNImputer
from ._iterative import IterativeImputer as IterativeImputer
import typing as typing

__all__ = ["MissingIndicator", "SimpleImputer", "KNNImputer"]



from typing import ClassVar, TypeVar
from numpy.random import RandomState
from .base import (
    ClassifierMixin,
    clone as clone,
    is_classifier as is_classifier,
    MultiOutputMixin,
    MetaEstimatorMixin,
    is_regressor as is_regressor,
)
from .preprocessing import LabelBinarizer
from .base import BaseEstimator
from .utils.validation import check_is_fitted as check_is_fitted
from .utils.multiclass import (
    check_classification_targets as check_classification_targets,
)
from numpy import ndarray
from numbers import Integral as Integral, Real as Real
from .utils.metaestimators import available_if as available_if
from .utils import check_random_state as check_random_state
from .utils._param_validation import HasMethods as HasMethods, Interval as Interval
from ._typing import Int, MatrixLike, ArrayLike, Float
from .metrics.pairwise import euclidean_distances as euclidean_distances
from .utils.parallel import delayed as delayed, Parallel as Parallel

OneVsRestClassifier_Self = TypeVar(
    "OneVsRestClassifier_Self", bound="OneVsRestClassifier"
)
OneVsOneClassifier_Self = TypeVar("OneVsOneClassifier_Self", bound="OneVsOneClassifier")
OutputCodeClassifier_Self = TypeVar(
    "OutputCodeClassifier_Self", bound="OutputCodeClassifier"
)


# Author: Mathieu Blondel <mathieu@mblondel.org>
# Author: Hamzeh Alsalhi <93hamsal@gmail.com>
#
# License: BSD 3 clause

import array
import numpy as np
import warnings
import scipy.sparse as sp
import itertools

__all__ = [
    "OneVsRestClassifier",
    "OneVsOneClassifier",
    "OutputCodeClassifier",
]


class _ConstantPredictor(BaseEstimator):
    def fit(self, X, y):
        ...

    def predict(self, X):
        ...

    def decision_function(self, X):
        ...

    def predict_proba(self, X):
        ...


class OneVsRestClassifier(
    MultiOutputMixin, ClassifierMixin, MetaEstimatorMixin, BaseEstimator
):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    label_binarizer_: LabelBinarizer = ...
    classes_: ndarray = ...
    estimators_: list[BaseEstimator] = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self, estimator: BaseEstimator, *, n_jobs: None | Int = None, verbose: Int = 0
    ) -> None:
        ...

    def fit(
        self: OneVsRestClassifier_Self,
        X: MatrixLike | ArrayLike,
        y: MatrixLike | ArrayLike,
    ) -> OneVsRestClassifier_Self:
        ...

    def partial_fit(
        self: OneVsRestClassifier_Self,
        X: MatrixLike | ArrayLike,
        y: MatrixLike | ArrayLike,
        classes: None | ArrayLike = None,
    ) -> OneVsRestClassifier_Self:
        ...

    def predict(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...

    def predict_proba(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...

    def decision_function(self, X: MatrixLike) -> ndarray:
        ...

    @property
    def multilabel_(self) -> bool:
        ...

    @property
    def n_classes_(self) -> int:
        ...


class OneVsOneClassifier(MetaEstimatorMixin, ClassifierMixin, BaseEstimator):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    pairwise_indices_: None | list = ...
    classes_: ndarray = ...
    estimators_: list[BaseEstimator] = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(self, estimator: BaseEstimator, *, n_jobs: None | Int = None) -> None:
        ...

    def fit(
        self: OneVsOneClassifier_Self, X: MatrixLike | ArrayLike, y: ArrayLike
    ) -> OneVsOneClassifier_Self:
        ...

    def partial_fit(
        self: OneVsOneClassifier_Self,
        X: MatrixLike,
        y: ArrayLike,
        classes: None | ArrayLike = None,
    ) -> OneVsOneClassifier_Self:
        ...

    def predict(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...

    def decision_function(self, X: MatrixLike) -> ndarray:
        ...

    @property
    def n_classes_(self) -> int:
        ...


class OutputCodeClassifier(MetaEstimatorMixin, ClassifierMixin, BaseEstimator):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    code_book_: ndarray = ...
    classes_: ndarray = ...
    estimators_: list[BaseEstimator] = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        estimator: BaseEstimator,
        *,
        code_size: Float = 1.5,
        random_state: RandomState | None | Int = None,
        n_jobs: None | Int = None
    ) -> None:
        ...

    def fit(
        self: OutputCodeClassifier_Self, X: MatrixLike | ArrayLike, y: ArrayLike
    ) -> OutputCodeClassifier_Self:
        ...

    def predict(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...



from collections import deque
from itertools import islice

from ...classes.graph import Graph

from ...utils import arbitrary_element
from .utils import (
    CurrentEdge,
    GlobalRelabelThreshold,
    Level,
    build_residual_network,
    detect_unboundedness,
)

__all__ = ["preflow_push"]

def preflow_push_impl(
    G: Graph, s, t, capacity, residual, global_relabel_freq, value_only
): ...
def preflow_push(
    G: Graph,
    s,
    t,
    capacity: str = "capacity",
    residual=None,
    global_relabel_freq=1,
    value_only: bool = False,
): ...



from typing import ClassVar, Literal, Mapping
from numpy.random import RandomState
from ._stochastic_gradient import BaseSGDClassifier
from numpy import ndarray
from ..utils._param_validation import StrOptions as StrOptions, Interval as Interval
from numbers import Real as Real
from .._typing import Float, Int
from ._sgd_fast import LossFunction

# Author: Mathieu Blondel
# License: BSD 3 clause


class Perceptron(BaseSGDClassifier):
    t_: int = ...
    n_iter_: int = ...
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    loss_function_: LossFunction = ...
    intercept_: ndarray = ...
    coef_: ndarray = ...
    classes_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        *,
        penalty: None | Literal["l2", "l1", "elasticnet"] = None,
        alpha: Float = 0.0001,
        l1_ratio: Float = 0.15,
        fit_intercept: bool = True,
        max_iter: Int = 1000,
        tol: None | Float = 1e-3,
        shuffle: bool = True,
        verbose: Int = 0,
        eta0: Float = 1.0,
        n_jobs: None | Int = None,
        random_state: RandomState | None | Int = 0,
        early_stopping: bool = False,
        validation_fraction: Float = 0.1,
        n_iter_no_change: Int = 5,
        class_weight: Mapping[str, float] | None | str = None,
        warm_start: bool = False,
    ) -> None:
        ...



from collections import deque
from operator import itemgetter

from ..classes.graph import Graph

from ..utils import arbitrary_element

__all__ = ["cuthill_mckee_ordering", "reverse_cuthill_mckee_ordering"]

def cuthill_mckee_ordering(G: Graph, heuristic=None): ...
def reverse_cuthill_mckee_ordering(G: Graph, heuristic=None): ...
def connected_cuthill_mckee_ordering(G: Graph, heuristic=None): ...
def pseudo_peripheral_node(G: Graph): ...



from vispy.util.svg.transform import Transform
from numpy.typing import ArrayLike, NDArray

# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2014, Nicolas P. Rougier
# Distributed under the (new) BSD License. See LICENSE.txt for more info.
# -----------------------------------------------------------------------------

import numpy as np
from ... import glsl
from .collection import Collection
from ..transforms import NullTransform

class RawPointCollection(Collection):
    def __init__(
        self,
        user_dtype: ArrayLike | None = None,
        transform: Transform | None = None,
        vertex: str | None = None,
        fragment: str | None = None,
        **kwargs,
    ): ...
    def append(self, P: NDArray, itemsize: int | None = None, **kwargs): ...



from enum import Enum
from typing import (
    Any,
    Callable,
    Dict,
    Iterator,
    List,
    Mapping,
    Optional,
    Sequence,
    Tuple,
    Type,
    Union,
)

from django.http.request import HttpRequest
from django.template.context import Context as Context
from django.template.engine import Engine
from django.template.library import Library
from django.template.loaders.base import Loader
from django.utils.safestring import SafeText

FILTER_SEPARATOR: str
FILTER_ARGUMENT_SEPARATOR: str
VARIABLE_ATTRIBUTE_SEPARATOR: str
BLOCK_TAG_START: str
BLOCK_TAG_END: str
VARIABLE_TAG_START: str
VARIABLE_TAG_END: str
COMMENT_TAG_START: str
COMMENT_TAG_END: str
TRANSLATOR_COMMENT_MARK: str
SINGLE_BRACE_START: str
SINGLE_BRACE_END: str
UNKNOWN_SOURCE: str
tag_re: Any
logger: Any

class TokenType(Enum):
    TEXT: int
    VAR: int
    BLOCK: int
    COMMENT: int

class VariableDoesNotExist(Exception):
    msg: str = ...
    params: Tuple[Union[Dict[str, str], str]] = ...
    def __init__(
        self, msg: str, params: Tuple[Union[Dict[str, str], str]] = ...
    ) -> None: ...

class Origin:
    name: str = ...
    template_name: Optional[Union[bytes, str]] = ...
    loader: Optional[Loader] = ...
    def __init__(
        self,
        name: str,
        template_name: Optional[Union[bytes, str]] = ...,
        loader: Optional[Loader] = ...,
    ) -> None: ...
    @property
    def loader_name(self) -> Optional[str]: ...

class Template:
    name: Optional[str] = ...
    origin: Origin = ...
    engine: Engine = ...
    source: str = ...
    nodelist: NodeList = ...
    def __init__(
        self,
        template_string: Union[Template, str],
        origin: Optional[Origin] = ...,
        name: Optional[str] = ...,
        engine: Optional[Engine] = ...,
    ) -> None: ...
    def __iter__(self) -> None: ...
    def render(
        self,
        context: Optional[Union[Context, Dict[str, Any]]] = ...,
        request: Optional[HttpRequest] = ...,
    ) -> Any: ...
    def compile_nodelist(self) -> NodeList: ...
    def get_exception_info(
        self, exception: Exception, token: Token
    ) -> Dict[str, Any]: ...

def linebreak_iter(template_source: str) -> Iterator[int]: ...

class Token:
    contents: str
    token_type: TokenType
    lineno: Optional[int] = ...
    position: Optional[Tuple[int, int]] = ...
    def __init__(
        self,
        token_type: TokenType,
        contents: str,
        position: Optional[Tuple[int, int]] = ...,
        lineno: Optional[int] = ...,
    ) -> None: ...
    def split_contents(self) -> List[str]: ...

class Lexer:
    template_string: str = ...
    verbatim: Union[bool, str] = ...
    def __init__(self, template_string: str) -> None: ...
    def tokenize(self) -> List[Token]: ...
    def create_token(
        self,
        token_string: str,
        position: Optional[Tuple[int, int]],
        lineno: int,
        in_tag: bool,
    ) -> Token: ...

class DebugLexer(Lexer):
    template_string: str
    verbatim: Union[bool, str]
    def tokenize(self) -> List[Token]: ...

class Parser:
    tokens: Union[List[Token], str] = ...
    tags: Dict[str, Callable[..., Any]] = ...
    filters: Dict[str, Callable[..., Any]] = ...
    command_stack: List[Tuple[str, Token]] = ...
    libraries: Dict[str, Library] = ...
    origin: Optional[Origin] = ...
    def __init__(
        self,
        tokens: Union[List[Token], str],
        libraries: Optional[Dict[str, Library]] = ...,
        builtins: Optional[List[Library]] = ...,
        origin: Optional[Origin] = ...,
    ) -> None: ...
    def parse(self, parse_until: Optional[Tuple[str, ...]] = ...) -> NodeList: ...
    def skip_past(self, endtag: str) -> None: ...
    def extend_nodelist(self, nodelist: NodeList, node: Node, token: Token) -> None: ...
    def error(self, token: Token, e: Union[Exception, str]) -> Exception: ...
    def invalid_block_tag(
        self,
        token: Token,
        command: str,
        parse_until: Union[List[Any], Tuple[str]] = ...,
    ) -> Any: ...
    def unclosed_block_tag(self, parse_until: Tuple[str]) -> Any: ...
    def next_token(self) -> Token: ...
    def prepend_token(self, token: Token) -> None: ...
    def delete_first_token(self) -> None: ...
    def add_library(self, lib: Library) -> None: ...
    def compile_filter(self, token: str) -> FilterExpression: ...
    def find_filter(self, filter_name: str) -> Callable[..., Any]: ...

constant_string: Any
filter_raw_string: Any
filter_re: Any

class FilterExpression:
    token: str = ...
    filters: List[Any] = ...
    var: Any = ...
    def __init__(self, token: str, parser: Parser) -> None: ...
    def resolve(
        self, context: Mapping[str, Any], ignore_failures: bool = ...
    ) -> Any: ...
    @staticmethod
    def args_check(
        name: str, func: Callable[..., Any], provided: List[Tuple[bool, Any]]
    ) -> bool: ...

class Variable:
    var: Union[Dict[Any, Any], str] = ...
    literal: Optional[Union[SafeText, float]] = ...
    lookups: Optional[Tuple[str]] = ...
    translate: bool = ...
    message_context: Optional[str] = ...
    def __init__(self, var: Union[Dict[Any, Any], str]) -> None: ...
    def resolve(
        self, context: Union[Mapping[str, Mapping[str, Any]], Context, int, str]
    ) -> Any: ...

class Node:
    must_be_first: bool = ...
    child_nodelists: Any = ...
    origin: Origin
    token: Token = ...
    def render(self, context: Context) -> str: ...
    def render_annotated(self, context: Context) -> Union[int, str]: ...
    def __iter__(self) -> None: ...
    def get_nodes_by_type(self, nodetype: Type[Node]) -> List[Node]: ...

class NodeList(List[Node]):
    contains_nontext: bool = ...
    def render(self, context: Context) -> SafeText: ...
    def get_nodes_by_type(self, nodetype: Type[Node]) -> List[Node]: ...

class TextNode(Node):
    s: str = ...
    def __init__(self, s: str) -> None: ...

def render_value_in_context(value: Any, context: Context) -> str: ...

class VariableNode(Node):
    filter_expression: FilterExpression = ...
    def __init__(self, filter_expression: FilterExpression) -> None: ...

kwarg_re: Any

def token_kwargs(
    bits: Sequence[str], parser: Parser, support_legacy: bool = ...
) -> Dict[str, FilterExpression]: ...



from typing import Any, Literal
from matplotlib.collections import QuadMesh
from matplotlib.axes import Axes
from matplotlib.contour import QuadContourSet
from functools import reduce as reduce
from ...utils.validation import check_is_fitted as check_is_fitted
from ...preprocessing import LabelEncoder as LabelEncoder
from ...utils import check_matplotlib_support as check_matplotlib_support
from ..._typing import MatrixLike, Int, Float
from ...base import is_regressor as is_regressor
from matplotlib.figure import Figure

import numpy as np


class DecisionBoundaryDisplay:
    figure_: Figure = ...
    ax_: Axes = ...
    surface_: QuadContourSet | QuadMesh = ...

    def __init__(
        self,
        *,
        xx0: MatrixLike,
        xx1: MatrixLike,
        response: MatrixLike,
        xlabel: None | str = None,
        ylabel: None | str = None,
    ) -> None:
        ...

    def plot(
        self,
        plot_method: Literal[
            "contourf", "contourf", "contour", "pcolormesh"
        ] = "contourf",
        ax: None | Axes = None,
        xlabel: None | str = None,
        ylabel: None | str = None,
        **kwargs,
    ) -> DecisionBoundaryDisplay:
        ...

    @classmethod
    def from_estimator(
        cls,
        estimator: Any,
        X: MatrixLike,
        *,
        grid_resolution: Int = 100,
        eps: Float = 1.0,
        plot_method: Literal[
            "contourf", "contour", "pcolormesh", "contourf"
        ] = "contourf",
        response_method: Literal[
            "auto", "predict_proba", "decision_function", "predict", "auto"
        ] = "auto",
        xlabel: None | str = None,
        ylabel: None | str = None,
        ax: None | Axes = None,
        **kwargs,
    ) -> DecisionBoundaryDisplay:
        ...



# Python: 3.8.2 (tags/v3.8.2:7b3ab59, Feb 25 2020, 23:03:10) [MSC v.1916 64 bit (AMD64)]
# Library: scipy, version: 1.6.2
# Module: scipy._lib._fpumode, version: unspecified
import typing
import builtins as _mod_builtins

__doc__: typing.Any
__file__: str
__name__: str
__package__: str
def get_fpu_mode() -> typing.Any:
    'get_fpu_mode()\n\nGet the current FPU control word, in a platform-dependent format.\nReturns None if not implemented on current platform.'
    ...

def __getattr__(name) -> typing.Any:
    ...




from sre_parse import State
from tkinter.tix import HList
from matplotlib.mathtext import MathtextBackend
from typing import Literal
from .font_manager import FontProperties
import enum
import functools

def get_unicode_index(symbol: str, math: bool = True) -> int: ...

class Fonts:
    def __init__(
        self, default_font_prop: FontProperties, mathtext_backend: MathtextBackend
    ) -> None: ...
    def get_kern(
        self,
        font1,
        fontclass1,
        sym1: str,
        fontsize1: float,
        font2,
        fontclass2: float,
        sym2: str,
        fontsize2: float,
        dpi: float,
    )-> float: ...
    def get_metrics(
        self, font: str, font_class: str, sym: str, fontsize: float, dpi: float
    ) -> object: ...
    def render_glyph(self, ox: float, oy: float, font, font_class, sym, fontsize: float, dpi: float)-> None: ...
    def render_rect_filled(self, x1:float, y1: float, x2: float, y2: float)-> None: ...
    def get_xheight(self, font, fontsize: float, dpi: float)-> float: ...
    def get_underline_thickness(self, font, fontsize: float, dpi: float) -> float: ...
    def get_used_characters(self): ...
    def get_sized_alternatives_for_symbol(self, fontname, sym)-> list[tuple]: ...

class TruetypeFonts(Fonts):
    def __init__(self, default_font_prop, mathtext_backend) -> None: ...
    def get_xheight(self, fontname, fontsize: float, dpi: float)-> float: ...
    def get_underline_thickness(self, font, fontsize: float, dpi: float)-> float: ...
    def get_kern(
        self,
        font1,
        fontclass1,
        sym1: str,
        fontsize1: float,
        font2,
        fontclass2: float,
        sym2: str,
        fontsize2: float,
        dpi: float,
    ): ...

class BakomaFonts(TruetypeFonts):
    def __init__(self, *args, **kwargs) -> None: ...
    def get_sized_alternatives_for_symbol(self, fontname, sym)-> list[tuple]: ...

class UnicodeFonts(TruetypeFonts):
    def __init__(self, *args, **kwargs) -> None: ...
    def get_sized_alternatives_for_symbol(self, fontname, sym)-> list[tuple]: ...

class DejaVuFonts(UnicodeFonts):
    def __init__(self, *args, **kwargs) -> None: ...

class DejaVuSerifFonts(DejaVuFonts): ...
class DejaVuSansFonts(DejaVuFonts): ...

class StixFonts(UnicodeFonts):
    def __init__(self, *args, **kwargs) -> None: ...
    @functools.lru_cache()
    def get_sized_alternatives_for_symbol(self, fontname, sym)-> list[tuple]: ...

class StixSansFonts(StixFonts): ...

SHRINK_FACTOR: float = ...
NUM_SIZE_LEVELS: int = ...

class FontConstantsBase:

    script_space: float = ...
    subdrop: float = ...
    sup1: float = ...
    sub1: float = ...
    sub2: float = ...
    delta: float = ...
    delta_slanted: float = ...
    delta_integral: float = ...

class ComputerModernFontConstants(FontConstantsBase):
    script_space: float = ...
    subdrop: float = ...
    sup1: float = ...
    sub1: float = ...
    sub2: float = ...
    delta: float = ...
    delta_slanted: float = ...
    delta_integral: float = ...

class STIXFontConstants(FontConstantsBase):
    script_space: float = ...
    sup1: float = ...
    sub2: float = ...
    delta: float = ...
    delta_slanted: float = ...
    delta_integral: float = ...

class STIXSansFontConstants(FontConstantsBase):
    script_space: float = ...
    sup1: float = ...
    delta_slanted: float = ...
    delta_integral: float = ...

class DejaVuSerifFontConstants(FontConstantsBase): ...
class DejaVuSansFontConstants(FontConstantsBase): ...

class Node:
    def __init__(self) -> None: ...
    def __repr__(self)-> str: ...
    def get_kerning(self, next)-> float: ...
    def shrink(self)-> None: ...
    def render(self, x: float, y: float)-> None: ...

class Box(Node):
    def __init__(self, width: float, height: float, depth: float) -> None: ...
    def shrink(self)-> None: ...
    def render(self, x1: float, y1: float, x2: float, y2: float)-> None: ...

class Vbox(Box):
    def __init__(self, height: float, depth: float) -> None: ...

class Hbox(Box):
    def __init__(self, width: float) -> None: ...

class Char(Node):
    def __init__(self, c, state) -> None: ...
    def __repr__(self)-> str: ...
    def is_slanted(self)-> bool: ...
    def get_kerning(self, next)-> float: ...
    def render(self, x: float, y: float)-> None: ...
    def shrink(self)-> None: ...

class Accent(Char):
    def shrink(self)-> None: ...
    def render(self, x: float, y: float)-> None: ...

class List(Box):
    def __init__(self, elements) -> None: ...
    def __repr__(self)-> str: ...
    def shrink(self)-> None: ...

class Hlist(List):
    def __init__(self, elements, w: float=0, m: Literal["exactly", "additional"] = 'additional', do_kern: bool=True) -> None: ...
    def kern(self)-> None: ...
    def hpack(self, w: float = 0, m: Literal["exactly", "additional"] = 'additional')-> None: ...

class Vlist(List):
    def __init__(self, elements, h=0, m=...) -> None: ...
    def vpack(
        self, h: float = ..., m: Literal["exactly", "additional"] = 'additional', l: float = ...
    )-> None: ...

class Rule(Box):
    def __init__(self, width: float, height: float, depth: float, state) -> None: ...
    def render(self, x: float, y: float, w: float, h: float)-> None: ...

class Hrule(Rule):
    def __init__(self, state, thickness=...) -> None: ...

class Vrule(Rule):
    def __init__(self, state) -> None: ...

class Glue(Node):
    def __init__(self, glue_type) -> None: ...
    def shrink(self): ...

class HCentered(Hlist):
    def __init__(self, elements) -> None: ...

class VCentered(Vlist):
    def __init__(self, elements) -> None: ...

class Kern(Node):

    height: float = ...
    depth: float = ...
    def __init__(self, width: float) -> None: ...
    def __repr__(self)-> str: ...
    def shrink(self)-> None: ...

class AutoHeightChar(Hlist):
    def __init__(self, c, height: float, depth: float, state, always=False, factor=None) -> None: ...

class AutoWidthChar(Hlist):
    def __init__(self, c, width: float, state, always=False, char_class=Char) -> None: ...

def ship(ox, oy, box)-> None: ...
def Error(msg): ...

class ParserState:
    def __init__(self, font_output, font, font_class, fontsize: float, dpi: float) -> None: ...
    def copy(self)-> ParserState: ...
    @property
    def font(self): ...
    @font.setter
    def font(self, name: str)-> None: ...
    def get_current_underline_thickness(self)-> float: ...

def cmd(expr, args): ...

class Parser:
    class _MathStyle(enum.Enum):
        DISPLAYSTYLE = ...
        TEXTSTYLE = ...
        SCRIPTSTYLE = ...
        SCRIPTSCRIPTSTYLE = ...
    def __init__(self) -> None: ...
    def parse(self, s, fonts_object, fontsize: float, dpi: float): ...
    def get_state(self) -> ParserState: ...
    def pop_state(self) -> ParserState: ...
    def push_state(self): ...
    def main(self, s, loc, toks)-> list[HList]: ...
    def math_string(self, s, loc, toks): ...
    def math(self, s, loc, toks)-> list[HList]: ...
    def non_math(self, s, loc, toks)-> list[HList]: ...

    float_literal = ...

    def space(self, s, loc, toks)-> list[Kern]: ...
    def customspace(self, s, loc, toks)-> list[Kern]: ...
    def symbol(self, s, loc, toks)-> list: ...

    accentprefixed = ...
    def unknown_symbol(self, s, loc, toks): ...
    def accent(self, s, loc, toks)-> Vlist: ...
    def function(self, s, loc, toks)-> Hlist: ...
    def operatorname(self, s, loc, toks)-> HList: ...
    def start_group(self, s, loc, toks)-> list: ...
    def group(self, s, loc, toks)-> list[Hlist]: ...
    def required_group(self, s, loc, toks)-> Hlist: ...

    optional_group = ...
    def end_group(self, s, loc, toks)-> list: ...
    def font(self, s, loc, toks)-> list: ...
    def is_overunder(self, nucleus)-> bool: ...
    def is_dropsub(self, nucleus)-> bool: ...
    def is_slanted(self, nucleus)-> bool: ...
    def is_between_brackets(self, s, loc): ...
    def subsuper(self, s, loc, toks): ...
    def style_literal(self, s, loc, toks): ...
    def genfrac(self, s, loc, toks): ...
    def frac(self, s, loc, toks): ...
    def dfrac(self, s, loc, toks): ...
    def binom(self, s, loc, toks): ...

    overset = ...
    def sqrt(self, s, loc, toks)-> list[Hlist]: ...
    def overline(self, s, loc, toks)-> list[Hlist]: ...
    def auto_delim(self, s, loc, toks)-> Hlist: ...



import itertools

from ...classes.graph import Graph

# Define the default maximum flow function to use in all flow based
# cut algorithms.
from ...algorithms.flow import build_residual_network, edmonds_karp

default_flow_func = ...

from .utils import build_auxiliary_edge_connectivity, build_auxiliary_node_connectivity

__all__ = [
    "minimum_st_node_cut",
    "minimum_node_cut",
    "minimum_st_edge_cut",
    "minimum_edge_cut",
]

def minimum_st_edge_cut(
    G: Graph, s, t, flow_func=None, auxiliary=None, residual=None
) -> set: ...
def minimum_st_node_cut(
    G: Graph, s, t, flow_func=None, auxiliary=None, residual=None
) -> set: ...
def minimum_node_cut(G: Graph, s=None, t=None, flow_func=None) -> set: ...
def minimum_edge_cut(G: Graph, s=None, t=None, flow_func=None) -> set: ...



from typing import Any

from itertools import chain

from ..classes.graph import Graph

__all__ = [
    "boundary_expansion",
    "conductance",
    "cut_size",
    "edge_expansion",
    "mixing_expansion",
    "node_expansion",
    "normalized_cut_size",
    "volume",
]

# TODO STILL NEED TO UPDATE ALL THE DOCUMENTATION!

def cut_size(G: Graph, S, T=None, weight: Any = None): ...
def volume(G: Graph, S, weight: Any = None): ...
def normalized_cut_size(G: Graph, S, T=None, weight: Any = None): ...
def conductance(G: Graph, S, T=None, weight: Any = None): ...
def edge_expansion(G: Graph, S, T=None, weight: Any = None): ...
def mixing_expansion(G: Graph, S, T=None, weight: Any = None): ...

# TODO What is the generalization to two arguments, S and T? Does the
# denominator become `min(len(S), len(T))`?
def node_expansion(G: Graph, S): ...

# TODO What is the generalization to two arguments, S and T? Does the
# denominator become `min(len(S), len(T))`?
def boundary_expansion(G: Graph, S): ...



import warnings

from ..utils import edges_equal, graphs_equal, nodes_equal

__all__ = [
    "assert_nodes_equal",
    "assert_edges_equal",
    "assert_graphs_equal",
    "almost_equal",
]

def almost_equal(x, y, places=7): ...
def assert_nodes_equal(nodes1, nodes2): ...
def assert_edges_equal(edges1, edges2): ...
def assert_graphs_equal(graph1, graph2): ...



# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

from .axis import AxisVisual as AxisVisual  # noqa
from .box import BoxVisual as BoxVisual  # noqa
from .cube import CubeVisual as CubeVisual  # noqa
from .ellipse import EllipseVisual as EllipseVisual  # noqa
from .gridlines import GridLinesVisual as GridLinesVisual  # noqa
from .image import ImageVisual as ImageVisual  # noqa
from .image_complex import ComplexImageVisual as ComplexImageVisual  # noqa
from .gridmesh import GridMeshVisual as GridMeshVisual  # noqa
from .histogram import HistogramVisual as HistogramVisual  # noqa
from .infinite_line import InfiniteLineVisual as InfiniteLineVisual  # noqa
from .isocurve import IsocurveVisual as IsocurveVisual  # noqa
from .isoline import IsolineVisual as IsolineVisual  # noqa
from .isosurface import IsosurfaceVisual as IsosurfaceVisual  # noqa
from .line import LineVisual as LineVisual, ArrowVisual as ArrowVisual  # noqa
from .linear_region import LinearRegionVisual as LinearRegionVisual  # noqa
from .line_plot import LinePlotVisual as LinePlotVisual  # noqa
from .markers import MarkersVisual as MarkersVisual  # noqa
from .mesh import MeshVisual as MeshVisual  # noqa
from .mesh_normals import MeshNormalsVisual as MeshNormalsVisual  # noqa
from .plane import PlaneVisual as PlaneVisual  # noqa
from .polygon import PolygonVisual as PolygonVisual  # noqa
from .rectangle import RectangleVisual as RectangleVisual  # noqa
from .regular_polygon import RegularPolygonVisual as RegularPolygonVisual  # noqa
from .scrolling_lines import ScrollingLinesVisual as ScrollingLinesVisual  # noqa
from .spectrogram import SpectrogramVisual as SpectrogramVisual  # noqa
from .sphere import SphereVisual as SphereVisual  # noqa
from .surface_plot import SurfacePlotVisual as SurfacePlotVisual  # noqa
from .text import TextVisual as TextVisual  # noqa
from .tube import TubeVisual as TubeVisual  # noqa
from .visual import (
    BaseVisual as BaseVisual,
    Visual as Visual,
    CompoundVisual as CompoundVisual,
)  # noqa
from .volume import VolumeVisual as VolumeVisual  # noqa
from .xyz_axis import XYZAxisVisual as XYZAxisVisual  # noqa
from .border import _BorderVisual as _BorderVisual  # noqa
from .colorbar import ColorBarVisual as ColorBarVisual  # noqa
from .graphs import GraphVisual as GraphVisual  # noqa
from .windbarb import WindbarbVisual as WindbarbVisual  # noqa



from typing import Any, Dict, List, NamedTuple, Optional, Set, Type

from django.db.backends.base.base import BaseDatabaseWrapper
from django.db.backends.utils import CursorWrapper
from django.db.models.base import Model

class TableInfo(NamedTuple):
    name: Any
    type: Any

class FieldInfo(NamedTuple):
    name: Any
    type_code: Any
    display_size: Any
    internal_size: Any
    precision: Any
    scale: Any
    null_ok: Any
    default: Any

class BaseDatabaseIntrospection:
    data_types_reverse: Any = ...
    connection: Any = ...
    def __init__(self, connection: BaseDatabaseWrapper) -> None: ...
    def get_field_type(self, data_type: str, description: FieldInfo) -> str: ...
    def table_name_converter(self, name: str) -> str: ...
    def column_name_converter(self, name: str) -> str: ...
    def table_names(
        self, cursor: Optional[CursorWrapper] = ..., include_views: bool = ...
    ) -> List[str]: ...
    def get_table_list(self, cursor: Any) -> None: ...
    def django_table_names(
        self, only_existing: bool = ..., include_views: bool = ...
    ) -> List[str]: ...
    def installed_models(self, tables: List[str]) -> Set[Type[Model]]: ...
    def sequence_list(self) -> List[Dict[str, str]]: ...
    def get_sequences(
        self, cursor: Any, table_name: Any, table_fields: Any = ...
    ) -> None: ...
    def get_key_columns(self, cursor: Any, table_name: Any) -> None: ...
    def get_primary_key_column(self, cursor: Any, table_name: Any) -> Any: ...
    def get_constraints(self, cursor: Any, table_name: Any) -> None: ...



from vispy.util.svg.color import Color

# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

from .image import ImageVisual
from ..color import Color
from .shaders import Function

_GRID_COLOR: str = ...

class GridLinesVisual(ImageVisual):
    def __init__(self, scale: tuple = ..., color: Color | str = "w"): ...
    @property
    def size(self): ...
    def _prepare_transforms(self, view): ...
    def _prepare_draw(self, view): ...



from ._variance_threshold import VarianceThreshold as VarianceThreshold
from ._rfe import RFE as RFE, RFECV as RFECV
from ._base import SelectorMixin as SelectorMixin
from ._from_model import SelectFromModel as SelectFromModel
from ._mutual_info import (
    mutual_info_regression as mutual_info_regression,
    mutual_info_classif as mutual_info_classif,
)
from ._sequential import SequentialFeatureSelector as SequentialFeatureSelector
from ._univariate_selection import (
    chi2 as chi2,
    f_classif as f_classif,
    f_oneway as f_oneway,
    f_regression as f_regression,
    r_regression as r_regression,
    SelectPercentile as SelectPercentile,
    SelectKBest as SelectKBest,
    SelectFpr as SelectFpr,
    SelectFdr as SelectFdr,
    SelectFwe as SelectFwe,
    GenericUnivariateSelect as GenericUnivariateSelect,
)


__all__ = [
    "GenericUnivariateSelect",
    "SequentialFeatureSelector",
    "RFE",
    "RFECV",
    "SelectFdr",
    "SelectFpr",
    "SelectFwe",
    "SelectKBest",
    "SelectFromModel",
    "SelectPercentile",
    "VarianceThreshold",
    "chi2",
    "f_classif",
    "f_oneway",
    "f_regression",
    "r_regression",
    "mutual_info_classif",
    "mutual_info_regression",
    "SelectorMixin",
]



from typing import Sequence, Callable
import bz2
import collections
import gzip
import inspect
import itertools
import re
from collections import defaultdict
from contextlib import contextmanager
from os.path import splitext
from pathlib import Path

from ..classes.graph import Graph
from ..utils import create_py_random_state, create_random_state

__all__ = [
    "not_implemented_for",
    "open_file",
    "nodes_or_number",
    "preserve_random_state",
    "random_state",
    "np_random_state",
    "py_random_state",
    "argmap",
]

def not_implemented_for(*graph_types): ...

# To handle new extensions, define a function accepting a `path` and `mode`.
# Then add the extension to _dispatch_dict.
fopeners: dict = ...
_dispatch_dict = ...  # type: ignore

def open_file(path_arg: str | int, mode: str = "r"): ...
def nodes_or_number(which_args: str | int | Sequence[str]): ...
def preserve_random_state(func): ...
def np_random_state(random_state_argument: str | int): ...
def random_state(random_state_argument): ...
def py_random_state(random_state_argument: str | int): ...

class argmap:
    def __init__(self, func: Callable, *args, try_finally=False): ...
    @staticmethod
    def _lazy_compile(func): ...
    def __call__(self, f: Callable) -> Callable: ...

    __count: int = ...

    @classmethod
    def _count(cls): ...

    _bad_chars = ...

    @classmethod
    def _name(cls, f): ...
    def compile(self, f: Callable) -> Callable: ...
    def assemble(self, f: Callable): ...
    @classmethod
    def signature(cls, f: Callable): ...

    Signature = ...

    @staticmethod
    def _flatten(nestlist, visited): ...

    _tabs = ...

    @staticmethod
    def _indent(*lines): ...



# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

from .base_filter import Filter
from ..shaders import Function, Varying
from ...color import colormap, Color

class IsolineFilter(Filter):
    FRAG_SHADER: str = ...

    def __init__(self, level=2.0, width=2.0, antialias=1.0, color="black", **kwargs): ...
    @property
    def level(self): ...
    @level.setter
    def level(self, lev): ...
    @property
    def width(self): ...
    @width.setter
    def width(self, w): ...
    @property
    def color(self): ...
    @color.setter
    def color(self, c): ...
    @property
    def antialias(self): ...
    @antialias.setter
    def antialias(self, a): ...

class Alpha(Filter):
    FRAG_SHADER: str = ...

    def __init__(self, alpha=1.0, **kwargs): ...
    @property
    def alpha(self): ...
    @alpha.setter
    def alpha(self, a): ...

class ColorFilter(Filter):
    FRAG_SHADER: str = ...

    def __init__(self, filter=..., fpos: int = 8, **kwargs): ...
    @property
    def filter(self): ...
    @filter.setter
    def filter(self, f): ...

class ZColormapFilter(Filter):
    FRAG_SHADER: str = ...
    VERT_SHADER: str = ...

    def __init__(self, cmap, zrange=..., fpos: int = 3, vpos: int = 9, **kwargs): ...
    def _attach(self, visual): ...



from typing import (
    Any,
    Callable,
    Dict,
    Generic,
    Iterable,
    List,
    Optional,
    Protocol,
    Sequence,
    Tuple,
    Type,
    TypeVar,
    Union,
    overload,
)
from uuid import UUID

from django.db import models
from django.db.models.base import Model
from django.db.models.deletion import Collector
from django.db.models.fields import _GT, _ST, Field
from django.db.models.fields.mixins import FieldCacheMixin
from django.db.models.fields.related_descriptors import (
    ForwardManyToOneDescriptor as ForwardManyToOneDescriptor,
)
from django.db.models.fields.related_descriptors import (  # noqa: F401
    ForwardOneToOneDescriptor as ForwardOneToOneDescriptor,
)
from django.db.models.fields.related_descriptors import (
    ManyToManyDescriptor as ManyToManyDescriptor,
)
from django.db.models.fields.related_descriptors import (
    ReverseManyToOneDescriptor as ReverseManyToOneDescriptor,
)
from django.db.models.fields.related_descriptors import (
    ReverseOneToOneDescriptor as ReverseOneToOneDescriptor,
)
from django.db.models.fields.reverse_related import (  # noqa: F401
    ForeignObjectRel as ForeignObjectRel,
)
from django.db.models.fields.reverse_related import ManyToManyRel as ManyToManyRel
from django.db.models.fields.reverse_related import ManyToOneRel as ManyToOneRel
from django.db.models.fields.reverse_related import OneToOneRel as OneToOneRel
from django.db.models.manager import ManyToManyRelatedManager
from django.db.models.query_utils import PathInfo, Q
from typing_extensions import Literal

class _DeleteProtocol(Protocol):
    def __call__(
        self,
        collector: Collector,
        field: Field[Any, Any],
        sub_objs: Sequence[Model],
        using: str,
    ) -> None: ...

_F = TypeVar("_F", bound=models.Field[Any, Any])
_Choice = Tuple[Any, str]
_ChoiceNamedGroup = Tuple[str, Iterable[_Choice]]
_FieldChoices = Iterable[Union[_Choice, _ChoiceNamedGroup]]
_ChoicesLimit = Union[Dict[str, Any], Q, Callable[[], Q]]
_OnDeleteOptions = Union[_DeleteProtocol, Callable[[Any], _DeleteProtocol]]

_ValidatorCallable = Callable[..., None]
_ErrorMessagesToOverride = Dict[str, Any]

RECURSIVE_RELATIONSHIP_CONSTANT: str = ...

class RelatedField(FieldCacheMixin, Generic[_ST, _GT], Field[_ST, _GT]):
    one_to_many: bool = ...
    one_to_one: bool = ...
    many_to_many: bool = ...
    many_to_one: bool = ...
    related_model: Type[_GT] = ...
    opts: Any = ...
    def get_forward_related_filter(self, obj: Model) -> Dict[str, Union[int, UUID]]: ...
    def get_reverse_related_filter(self, obj: Model) -> Q: ...
    @property
    def swappable_setting(self) -> Optional[str]: ...
    def set_attributes_from_rel(self) -> None: ...
    def do_related_class(self, other: Type[Model], cls: Type[Model]) -> None: ...
    def get_limit_choices_to(self) -> Dict[str, int]: ...
    def related_query_name(self) -> str: ...
    @property
    def target_field(self) -> Field[Any, Any]: ...

_M = TypeVar("_M", bound=Optional[Model])

class ForeignObject(Generic[_M], RelatedField[_M, _M]):
    one_to_many: Literal[False] = ...
    one_to_one: Literal[False] = ...
    many_to_many: Literal[False] = ...
    many_to_one: Literal[True] = ...
    related_model: Type[_M] = ...
    @overload
    def __new__(
        cls,
        to: Union[Type[_M], str],
        on_delete: _OnDeleteOptions,
        from_fields: Sequence[str],
        to_fields: Sequence[str],
        rel: Optional[ForeignObjectRel] = ...,
        related_name: Optional[str] = ...,
        related_query_name: Optional[str] = ...,
        limit_choices_to: Optional[_ChoicesLimit] = ...,
        parent_link: bool = ...,
        db_constraint: bool = ...,
        swappable: bool = ...,
        verbose_name: Optional[str] = ...,
        name: Optional[str] = ...,
        primary_key: bool = ...,
        max_length: Optional[int] = ...,
        unique: bool = ...,
        blank: bool = ...,
        null: Literal[False] = ...,
        db_index: bool = ...,
        default: Optional[Union[_M, Callable[[], _M]]] = ...,
        editable: bool = ...,
        auto_created: bool = ...,
        serialize: bool = ...,
        unique_for_date: Optional[str] = ...,
        unique_for_month: Optional[str] = ...,
        unique_for_year: Optional[str] = ...,
        choices: Iterable[
            Union[Tuple[_M, str], Tuple[str, Iterable[Tuple[_M, str]]]]
        ] = ...,
        help_text: str = ...,
        db_column: Optional[str] = ...,
        db_tablespace: Optional[str] = ...,
        validators: Iterable[_ValidatorCallable] = ...,
        error_messages: Optional[_ErrorMessagesToOverride] = ...,
    ) -> ForeignObject[_M]: ...
    @overload
    def __new__(
        cls,
        to: Union[Type[_M], str],
        on_delete: _OnDeleteOptions,
        from_fields: Sequence[str],
        to_fields: Sequence[str],
        rel: Optional[ForeignObjectRel] = ...,
        related_name: Optional[str] = ...,
        related_query_name: Optional[str] = ...,
        limit_choices_to: Optional[_ChoicesLimit] = ...,
        parent_link: bool = ...,
        db_constraint: bool = ...,
        swappable: bool = ...,
        verbose_name: Optional[str] = ...,
        name: Optional[str] = ...,
        primary_key: bool = ...,
        max_length: Optional[int] = ...,
        unique: bool = ...,
        blank: bool = ...,
        null: Literal[True] = ...,
        db_index: bool = ...,
        default: Union[_M, Callable[[], _M]] = ...,
        editable: bool = ...,
        auto_created: bool = ...,
        serialize: bool = ...,
        unique_for_date: Optional[str] = ...,
        unique_for_month: Optional[str] = ...,
        unique_for_year: Optional[str] = ...,
        choices: Iterable[
            Union[Tuple[_M, str], Tuple[str, Iterable[Tuple[_M, str]]]]
        ] = ...,
        help_text: str = ...,
        db_column: Optional[str] = ...,
        db_tablespace: Optional[str] = ...,
        validators: Iterable[_ValidatorCallable] = ...,
        error_messages: Optional[_ErrorMessagesToOverride] = ...,
    ) -> ForeignObject[Optional[_M]]: ...

class ForeignKey(Generic[_M], ForeignObject[_M]):
    one_to_many: Literal[False] = ...
    one_to_one: Literal[False] = ...
    many_to_many: Literal[False] = ...
    many_to_one: Literal[True] = ...
    related_model: Type[_M] = ...
    @overload
    def __new__(
        cls,
        to: Union[Type[_M], str],
        on_delete: _OnDeleteOptions,
        to_field: Optional[str] = ...,
        related_name: Optional[str] = ...,
        related_query_name: Optional[str] = ...,
        limit_choices_to: Optional[_ChoicesLimit] = ...,
        parent_link: bool = ...,
        db_constraint: bool = ...,
        swappable: bool = ...,
        verbose_name: Optional[str] = ...,
        name: Optional[str] = ...,
        primary_key: bool = ...,
        max_length: Optional[int] = ...,
        unique: bool = ...,
        blank: bool = ...,
        null: Literal[False] = ...,
        db_index: bool = ...,
        default: Optional[Union[_M, Callable[[], _M]]] = ...,
        editable: bool = ...,
        auto_created: bool = ...,
        serialize: bool = ...,
        unique_for_date: Optional[str] = ...,
        unique_for_month: Optional[str] = ...,
        unique_for_year: Optional[str] = ...,
        choices: Iterable[
            Union[Tuple[_M, str], Tuple[str, Iterable[Tuple[_M, str]]]]
        ] = ...,
        help_text: str = ...,
        db_column: Optional[str] = ...,
        db_tablespace: Optional[str] = ...,
        validators: Iterable[_ValidatorCallable] = ...,
        error_messages: Optional[_ErrorMessagesToOverride] = ...,
    ) -> ForeignKey[_M]: ...
    @overload
    def __new__(
        cls,
        to: Union[Type[_M], str],
        on_delete: _OnDeleteOptions,
        to_field: Optional[str] = ...,
        related_name: Optional[str] = ...,
        related_query_name: Optional[str] = ...,
        limit_choices_to: Optional[_ChoicesLimit] = ...,
        parent_link: bool = ...,
        db_constraint: bool = ...,
        swappable: bool = ...,
        verbose_name: Optional[str] = ...,
        name: Optional[str] = ...,
        primary_key: bool = ...,
        max_length: Optional[int] = ...,
        unique: bool = ...,
        blank: bool = ...,
        null: Literal[True] = ...,
        db_index: bool = ...,
        default: Union[_M, Callable[[], _M]] = ...,
        editable: bool = ...,
        auto_created: bool = ...,
        serialize: bool = ...,
        unique_for_date: Optional[str] = ...,
        unique_for_month: Optional[str] = ...,
        unique_for_year: Optional[str] = ...,
        choices: Iterable[
            Union[Tuple[_M, str], Tuple[str, Iterable[Tuple[_M, str]]]]
        ] = ...,
        help_text: str = ...,
        db_column: Optional[str] = ...,
        db_tablespace: Optional[str] = ...,
        validators: Iterable[_ValidatorCallable] = ...,
        error_messages: Optional[_ErrorMessagesToOverride] = ...,
    ) -> ForeignKey[Optional[_M]]: ...
    # class access
    @overload
    def __get__(self, instance: None, owner: Any) -> ForwardManyToOneDescriptor: ...
    # Model instance access
    @overload
    def __get__(self: ForeignKey[_M], instance: Any, owner: Any) -> _M: ...
    @overload
    def __get__(
        self: ForeignKey[Optional[_M]], instance: Any, owner: Any
    ) -> Optional[_M]: ...
    # non-Model instances
    @overload
    def __get__(self: _F, instance: Any, owner: Any) -> _F: ...

class OneToOneField(Generic[_M], ForeignKey[_M]):
    one_to_many: Literal[False] = ...
    one_to_one: Literal[True] = ...  # type: ignore [assignment]
    many_to_many: Literal[False] = ...
    many_to_one: Literal[False] = ...  # type: ignore [assignment]
    related_model: Type[_M] = ...
    @overload
    def __new__(
        cls,
        to: Union[Type[_M], str],
        on_delete: _OnDeleteOptions,
        to_field: Optional[str] = ...,
        related_name: Optional[str] = ...,
        related_query_name: Optional[str] = ...,
        limit_choices_to: Optional[_ChoicesLimit] = ...,
        parent_link: bool = ...,
        db_constraint: bool = ...,
        swappable: bool = ...,
        verbose_name: Optional[str] = ...,
        name: Optional[str] = ...,
        primary_key: bool = ...,
        max_length: Optional[int] = ...,
        unique: Literal[True] = ...,
        blank: bool = ...,
        null: Literal[False] = ...,
        db_index: bool = ...,
        default: Optional[Union[_M, Callable[[], _M]]] = ...,
        editable: bool = ...,
        auto_created: bool = ...,
        serialize: bool = ...,
        unique_for_date: Optional[str] = ...,
        unique_for_month: Optional[str] = ...,
        unique_for_year: Optional[str] = ...,
        choices: Iterable[
            Union[Tuple[_M, str], Tuple[str, Iterable[Tuple[_M, str]]]]
        ] = ...,
        help_text: str = ...,
        db_column: Optional[str] = ...,
        db_tablespace: Optional[str] = ...,
        validators: Iterable[_ValidatorCallable] = ...,
        error_messages: Optional[_ErrorMessagesToOverride] = ...,
    ) -> OneToOneField[_M]: ...
    @overload
    def __new__(
        cls,
        to: Union[Type[_M], str],
        on_delete: _OnDeleteOptions,
        to_field: Optional[str] = ...,
        related_name: Optional[str] = ...,
        related_query_name: Optional[str] = ...,
        limit_choices_to: Optional[_ChoicesLimit] = ...,
        parent_link: bool = ...,
        db_constraint: bool = ...,
        swappable: bool = ...,
        verbose_name: Optional[str] = ...,
        name: Optional[str] = ...,
        primary_key: bool = ...,
        max_length: Optional[int] = ...,
        unique: Literal[True] = ...,
        blank: bool = ...,
        null: Literal[True] = ...,
        db_index: bool = ...,
        default: Union[_M, Callable[[], _M]] = ...,
        editable: bool = ...,
        auto_created: bool = ...,
        serialize: bool = ...,
        unique_for_date: Optional[str] = ...,
        unique_for_month: Optional[str] = ...,
        unique_for_year: Optional[str] = ...,
        choices: Iterable[
            Union[Tuple[_M, str], Tuple[str, Iterable[Tuple[_M, str]]]]
        ] = ...,
        help_text: str = ...,
        db_column: Optional[str] = ...,
        db_tablespace: Optional[str] = ...,
        validators: Iterable[_ValidatorCallable] = ...,
        error_messages: Optional[_ErrorMessagesToOverride] = ...,
    ) -> OneToOneField[Optional[_M]]: ...
    # class access
    @overload
    def __get__(self, instance: None, owner: Any) -> ForwardOneToOneDescriptor: ...
    # Model instance access
    @overload
    def __get__(self: OneToOneField[_M], instance: Any, owner: Any) -> _M: ...
    @overload
    def __get__(
        self: OneToOneField[Optional[_M]], instance: Any, owner: Any
    ) -> Optional[_M]: ...
    # non-Model instances
    @overload
    def __get__(self: _F, instance: Any, owner: Any) -> _F: ...

_MM = TypeVar("_MM", bound=Model)
_MN = TypeVar("_MN", bound=Model)

class ManyToManyField(
    Generic[_MM, _MN], RelatedField[Sequence[_MN], ManyToManyRelatedManager[_MM, _MN]]
):

    one_to_many: Literal[False] = ...
    one_to_one: Literal[False] = ...
    many_to_many: Literal[True] = ...
    many_to_one: Literal[False] = ...
    rel_class: Any = ...
    description: Any = ...
    has_null_arg: Any = ...
    swappable: bool = ...
    related_model: Type[_MM] = ...  # type: ignore [assignment]
    through: Type[_MN]
    def __new__(
        cls,
        to: Union[Type[_MM], str],
        through: Union[Type[_MN], str] = ...,
        to_field: Optional[str] = ...,
        related_name: Optional[str] = ...,
        related_query_name: Optional[str] = ...,
        limit_choices_to: Optional[_ChoicesLimit] = ...,
        symmetrical: Optional[bool] = ...,
        through_fields: Optional[Tuple[str, str]] = ...,
        db_constraint: bool = ...,
        swappable: bool = ...,
        verbose_name: Optional[str] = ...,
        name: Optional[str] = ...,
        primary_key: bool = ...,
        max_length: Optional[int] = ...,
        unique: bool = ...,
        blank: bool = ...,
        null: bool = ...,
        db_index: bool = ...,
        default: Any = ...,
        editable: bool = ...,
        auto_created: bool = ...,
        serialize: bool = ...,
        unique_for_date: Optional[str] = ...,
        unique_for_month: Optional[str] = ...,
        unique_for_year: Optional[str] = ...,
        choices: Optional[_FieldChoices] = ...,
        help_text: str = ...,
        db_column: Optional[str] = ...,
        db_table: Optional[str] = ...,
        db_tablespace: Optional[str] = ...,
        validators: Iterable[_ValidatorCallable] = ...,
        error_messages: Optional[_ErrorMessagesToOverride] = ...,
    ) -> ManyToManyField[_MM, _MN]: ...
    def get_path_info(self, filtered_relation: None = ...) -> List[PathInfo]: ...
    def get_reverse_path_info(
        self, filtered_relation: None = ...
    ) -> List[PathInfo]: ...
    def contribute_to_related_class(
        self, cls: Type[Model], related: RelatedField[Any, Any]
    ) -> None: ...
    def m2m_db_table(self) -> str: ...
    def m2m_column_name(self) -> str: ...
    def m2m_reverse_name(self) -> str: ...
    def m2m_reverse_field_name(self) -> str: ...
    def m2m_target_field_name(self) -> str: ...
    def m2m_reverse_target_field_name(self) -> str: ...

def create_many_to_many_intermediary_model(
    field: Type[Field[Any, Any]], klass: Type[Model]
) -> Type[Model]: ...



# flake8: noqa

# This minimal dataset was available as part of
# scikit-image 0.15 and will be retained until
# further notice.
# Testing data and additional datasets should only
# be made available by pooch
legacy_datasets: list = ...

# Registry of datafiles that can be downloaded along with their SHA256 hashes
# To generate the SHA256 hash, use the command
# openssl sha256 filename
registry: dict = ...

registry_urls: dict = ...

legacy_registry: dict = ...



from vispy.util.svg.color import Color
from numpy.typing import NDArray, ArrayLike

# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

from __future__ import division

from .mesh import MeshVisual
from ..geometry.isosurface import isosurface
from ..color import Color

class IsosurfaceVisual(MeshVisual):
    def __init__(
        self,
        data: NDArray | None = None,
        level: None | float = None,
        vertex_colors: NDArray | None = None,
        face_colors: NDArray | None = None,
        color: NDArray | None = ...,
        **kwargs,
    ): ...
    @property
    def level(self): ...
    @level.setter
    def level(self, level): ...
    def set_data(
        self,
        data: NDArray | None = None,
        vertex_colors: ArrayLike | None = None,
        face_colors: ArrayLike | None = None,
        color: Color | None = None,
    ): ...
    def _prepare_draw(self, view): ...



"""
This type stub file was generated by pyright.
"""

from IPython.core.magic import Magics, cell_magic, magics_class
from IPython.core import magic_arguments

"""Simple magics for display formats"""
@magics_class
class DisplayMagics(Magics):
    """Magics for displaying various output types with literals

    Defines javascript/latex/svg/html cell magics for writing
    blocks in those languages, to be rendered in the frontend.
    """
    @cell_magic
    def js(self, line, cell): # -> None:
        """Run the cell block of Javascript code

        Alias of `%%javascript`

        Starting with IPython 8.0 %%javascript is pending deprecation to be replaced
        by a more flexible system

        Please See https://github.com/ipython/ipython/issues/13376
        """
        ...
    
    @cell_magic
    def javascript(self, line, cell): # -> None:
        """Run the cell block of Javascript code

        Starting with IPython 8.0 %%javascript is pending deprecation to be replaced
        by a more flexible system

        Please See https://github.com/ipython/ipython/issues/13376
        """
        ...
    
    @cell_magic
    def latex(self, line, cell): # -> None:
        """Render the cell as a block of LaTeX

        The subset of LaTeX which is supported depends on the implementation in
        the client.  In the Jupyter Notebook, this magic only renders the subset
        of LaTeX defined by MathJax
        [here](https://docs.mathjax.org/en/v2.5-latest/tex.html)."""
        ...
    
    @cell_magic
    def svg(self, line, cell): # -> None:
        """Render the cell as an SVG literal"""
        ...
    
    @magic_arguments.magic_arguments()
    @magic_arguments.argument('--isolated', action='store_true', default=False, help="""Annotate the cell as 'isolated'.
Isolated cells are rendered inside their own <iframe> tag""")
    @cell_magic
    def html(self, line, cell): # -> None:
        """Render the cell as a block of HTML"""
        ...
    
    @cell_magic
    def markdown(self, line, cell): # -> None:
        """Render the cell as Markdown text block"""
        ...
    





from typing import Any

from django.db import models
from psycopg2.extras import DateRange, DateTimeTZRange, NumericRange

class RangeField(models.Field[Any, Any]):
    empty_strings_allowed: bool = ...
    base_field: Any = ...
    range_type: Any = ...
    def get_prep_value(self, value: Any) -> Any: ...
    def to_python(self, value: Any) -> Any: ...
    def value_to_string(self, obj: Any) -> Any: ...

class IntegerRangeField(RangeField):
    def __get__(self, instance: Any, owner: Any) -> NumericRange: ...

class BigIntegerRangeField(RangeField):
    def __get__(self, instance: Any, owner: Any) -> NumericRange: ...

class DecimalRangeField(RangeField):
    def __get__(self, instance: Any, owner: Any) -> NumericRange: ...

class FloatRangeField(RangeField):
    def __get__(self, instance: Any, owner: Any) -> NumericRange: ...

class DateTimeRangeField(RangeField):
    def __get__(self, instance: Any, owner: Any) -> DateTimeTZRange: ...

class DateRangeField(RangeField):
    def __get__(self, instance: Any, owner: Any) -> DateRange: ...

class RangeOperators:
    EQUAL: str
    NOT_EQUAL: str
    CONTAINS: str
    CONTAINED_BY: str
    OVERLAPS: str
    FULLY_LT: str
    FULLY_GT: str
    NOT_LT: str
    NOT_GT: str
    ADJACENT_TO: str

class RangeBoundary(models.Expression):
    lower: str
    upper: str
    def __init__(
        self, inclusive_lower: bool = ..., inclusive_upper: bool = ...
    ) -> None: ...



from typing import (
    Any,
    Callable,
    Coroutine,
    Dict,
    List,
    Optional,
    Tuple,
    Union,
    overload,
)

from ..conf.urls import IncludedURLConf
from ..http.response import HttpResponseBase
from .resolvers import URLPattern, URLResolver

_ResponseType = Union[
    HttpResponseBase, Coroutine[Any, Any, HttpResponseBase], Coroutine[Any, Any, None]
]

def include(
    arg: Any, namespace: Optional[str] = ...
) -> Tuple[List[URLResolver], Optional[str], Optional[str]]: ...

# path()
@overload
def path(
    route: str,
    view: Callable[..., _ResponseType],
    kwargs: Dict[str, Any] = ...,
    name: str = ...,
) -> URLPattern: ...
@overload
def path(
    route: str, view: IncludedURLConf, kwargs: Dict[str, Any] = ..., name: str = ...
) -> URLResolver: ...
@overload
def path(
    route: str,
    view: List[Union[URLResolver, str]],
    kwargs: Dict[str, Any] = ...,
    name: str = ...,
) -> URLResolver: ...

# re_path()
@overload
def re_path(
    route: str,
    view: Callable[..., _ResponseType],
    kwargs: Dict[str, Any] = ...,
    name: str = ...,
) -> URLPattern: ...
@overload
def re_path(
    route: str, view: IncludedURLConf, kwargs: Dict[str, Any] = ..., name: str = ...
) -> URLResolver: ...
@overload
def re_path(
    route: str,
    view: List[Union[URLResolver, str]],
    kwargs: Dict[str, Any] = ...,
    name: str = ...,
) -> URLResolver: ...



from vispy.util.svg.color import Color
from numpy.typing import ArrayLike

# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.
import numbers

import numpy as np

from vispy.gloo import Texture2D, VertexBuffer
from vispy.visuals.shaders import Function, Varying
from vispy.visuals.filters import Filter
from ...color import Color

class TextureFilter(Filter):
    def __init__(self, texture: tuple | ArrayLike, texcoords: ArrayLike, enabled: bool = True): ...
    @property
    def enabled(self): ...
    @enabled.setter
    def enabled(self, enabled): ...
    @property
    def texture(self): ...
    @texture.setter
    def texture(self, texture): ...
    @property
    def texcoords(self): ...
    @texcoords.setter
    def texcoords(self, texcoords): ...
    def _update_texcoords_buffer(self, texcoords): ...
    def _attach(self, visual): ...

shading_vertex_template: str = ...

shading_fragment_template: str = ...  # noqa

def _as_rgba(intensity_or_color, default_rgb=(1.0, 1.0, 1.0)): ...

class ShadingFilter(Filter):
    def __init__(
        self,
        shading: str = "flat",
        ambient_coefficient: str | tuple | Color = ...,
        diffuse_coefficient: str | tuple | Color = ...,
        specular_coefficient: str | tuple | Color = ...,
        shininess: float = 100,
        light_dir: ArrayLike = ...,
        ambient_light: str | tuple | Color = ...,
        diffuse_light: str | tuple | Color = ...,
        specular_light: str | tuple | Color = ...,
        enabled: bool = True,
    ): ...
    @property
    def enabled(self): ...
    @enabled.setter
    def enabled(self, enabled): ...
    @property
    def shading(self): ...
    @shading.setter
    def shading(self, shading): ...
    @property
    def light_dir(self): ...
    @light_dir.setter
    def light_dir(self, direction): ...
    @property
    def ambient_light(self): ...
    @ambient_light.setter
    def ambient_light(self, light_color): ...
    @property
    def diffuse_light(self): ...
    @diffuse_light.setter
    def diffuse_light(self, light_color): ...
    @property
    def specular_light(self): ...
    @specular_light.setter
    def specular_light(self, light_color): ...
    @property
    def ambient_coefficient(self): ...
    @ambient_coefficient.setter
    def ambient_coefficient(self, color): ...
    @property
    def diffuse_coefficient(self): ...
    @diffuse_coefficient.setter
    def diffuse_coefficient(self, diffuse_coefficient): ...
    @property
    def specular_coefficient(self): ...
    @specular_coefficient.setter
    def specular_coefficient(self, specular_coefficient): ...
    @property
    def shininess(self): ...
    @shininess.setter
    def shininess(self, shininess): ...
    def _update_data(self): ...
    def on_mesh_data_updated(self, event): ...
    def _attach(self, visual): ...
    def _detach(self, visual): ...

wireframe_vertex_template: str = ...  # noqa

wireframe_fragment_template: str = ...  # noqa

class WireframeFilter(Filter):
    def __init__(
        self,
        enabled: bool = True,
        color: str | tuple | Color = "black",
        width: float = 1.0,
        wireframe_only=False,
        faces_only=False,
    ): ...
    @property
    def enabled(self): ...
    @enabled.setter
    def enabled(self, enabled): ...
    @property
    def color(self): ...
    @color.setter
    def color(self, color): ...
    @property
    def width(self): ...
    @width.setter
    def width(self, width): ...
    @property
    def wireframe_only(self): ...
    @wireframe_only.setter
    def wireframe_only(self, wireframe_only): ...
    @property
    def faces_only(self): ...
    @faces_only.setter
    def faces_only(self, faces_only): ...
    def _update_data(self): ...
    def on_mesh_data_updated(self, event): ...
    def _attach(self, visual): ...
    def _detach(self, visual): ...



"""
This type stub file was generated by pyright.
"""

from typing import Optional

"""sys.excepthook for IPython itself, leaves a detailed report on disk.

Authors:

* Fernando Perez
* Brian E. Granger
"""
_default_message_template = ...
_lite_message_template = ...
class CrashHandler:
    """Customizable crash handlers for IPython applications.

    Instances of this class provide a :meth:`__call__` method which can be
    used as a ``sys.excepthook``.  The :meth:`__call__` signature is::

        def __call__(self, etype, evalue, etb)
    """
    message_template = ...
    section_sep = ...
    def __init__(self, app, contact_name: Optional[str] = ..., contact_email: Optional[str] = ..., bug_tracker: Optional[str] = ..., show_crash_traceback: bool = ..., call_pdb: bool = ...) -> None:
        """Create a new crash handler

        Parameters
        ----------
        app : Application
            A running :class:`Application` instance, which will be queried at
            crash time for internal information.
        contact_name : str
            A string with the name of the person to contact.
        contact_email : str
            A string with the email address of the contact.
        bug_tracker : str
            A string with the URL for your project's bug tracker.
        show_crash_traceback : bool
            If false, don't print the crash traceback on stderr, only generate
            the on-disk report
        call_pdb
            Whether to call pdb on crash

        Attributes
        ----------
        These instances contain some non-argument attributes which allow for
        further customization of the crash handler's behavior. Please see the
        source for further details.

        """
        ...
    
    def __call__(self, etype, evalue, etb): # -> None:
        """Handle an exception, call for compatible with sys.excepthook"""
        ...
    
    def make_report(self, traceback):
        """Return a string containing a crash report."""
        ...
    


def crash_handler_lite(etype, evalue, tb): # -> None:
    """a light excepthook, adding a small message to the usual traceback"""
    ...




"""
This type stub file was generated by pyright.
"""

from IPython.core import magic_arguments
from IPython.core.magic import Magics, line_magic, magics_class
from IPython.terminal.interactiveshell import TerminalInteractiveShell
from typing import Set

"""
An embedded IPython shell.
"""
class KillEmbedded(Exception):
    ...


KillEmbeded = KillEmbedded
@magics_class
class EmbeddedMagics(Magics):
    @line_magic
    @magic_arguments.magic_arguments()
    @magic_arguments.argument('-i', '--instance', action='store_true', help='Kill instance instead of call location')
    @magic_arguments.argument('-x', '--exit', action='store_true', help='Also exit the current session')
    @magic_arguments.argument('-y', '--yes', action='store_true', help='Do not ask confirmation')
    def kill_embedded(self, parameter_s=...): # -> None:
        """%kill_embedded : deactivate for good the current embedded IPython

        This function (after asking for confirmation) sets an internal flag so
        that an embedded IPython will never activate again for the given call
        location. This is useful to permanently disable a shell that is being
        called inside a loop: once you've figured out what you needed from it,
        you may then kill it and the program will then continue to run without
        the interactive shell interfering again.

        Kill Instance Option:

            If for some reasons you need to kill the location where the instance
            is created and not called, for example if you create a single
            instance in one place and debug in many locations, you can use the
            ``--instance`` option to kill this specific instance. Like for the
            ``call location`` killing an "instance" should work even if it is
            recreated within a loop.

        .. note::

            This was the default behavior before IPython 5.2

        """
        ...
    
    @line_magic
    def exit_raise(self, parameter_s=...): # -> None:
        """%exit_raise Make the current embedded kernel exit and raise and exception.

        This function sets an internal flag so that an embedded IPython will
        raise a `IPython.terminal.embed.KillEmbedded` Exception on exit, and then exit the current I. This is
        useful to permanently exit a loop that create IPython embed instance.
        """
        ...
    


class _Sentinel:
    def __init__(self, repr) -> None:
        ...
    
    def __repr__(self):
        ...
    


class InteractiveShellEmbed(TerminalInteractiveShell):
    dummy_mode = ...
    exit_msg = ...
    embedded = ...
    should_raise = ...
    display_banner = ...
    exit_msg = ...
    term_title = ...
    _inactive_locations: Set[str] = ...
    @property
    def embedded_active(self):
        ...
    
    @embedded_active.setter
    def embedded_active(self, value): # -> None:
        ...
    
    def __init__(self, **kw) -> None:
        ...
    
    def init_sys_modules(self): # -> None:
        """
        Explicitly overwrite :mod:`IPython.core.interactiveshell` to do nothing.
        """
        ...
    
    def init_magics(self): # -> None:
        ...
    
    def __call__(self, header=..., local_ns=..., module=..., dummy=..., stack_depth=..., compile_flags=..., **kw): # -> None:
        """Activate the interactive interpreter.

        __call__(self,header='',local_ns=None,module=None,dummy=None) -> Start
        the interpreter shell with the given local and global namespaces, and
        optionally print a header string at startup.

        The shell can be globally activated/deactivated using the
        dummy_mode attribute. This allows you to turn off a shell used
        for debugging globally.

        However, *each* time you call the shell you can override the current
        state of dummy_mode with the optional keyword parameter 'dummy'. For
        example, if you set dummy mode on with IPShell.dummy_mode = True, you
        can still have a specific call work by making it as IPShell(dummy=False).
        """
        ...
    
    def mainloop(self, local_ns=..., module=..., stack_depth=..., compile_flags=...): # -> None:
        """Embeds IPython into a running python program.

        Parameters
        ----------
        local_ns, module
            Working local namespace (a dict) and module (a module or similar
            object). If given as None, they are automatically taken from the scope
            where the shell was called, so that program variables become visible.
        stack_depth : int
            How many levels in the stack to go to looking for namespaces (when
            local_ns or module is None). This allows an intermediate caller to
            make sure that this function gets the namespace from the intended
            level in the stack. By default (0) it will get its locals and globals
            from the immediate caller.
        compile_flags
            A bit field identifying the __future__ features
            that are enabled, as passed to the builtin :func:`compile` function.
            If given as None, they are automatically taken from the scope where
            the shell was called.

        """
        ...
    


def embed(*, header=..., compile_flags=..., **kwargs): # -> None:
    """Call this to embed IPython at the current point in your program.

    The first invocation of this will create a :class:`terminal.embed.InteractiveShellEmbed`
    instance and then call it.  Consecutive calls just call the already
    created instance.

    If you don't want the kernel to initialize the namespace
    from the scope of the surrounding function,
    and/or you want to load full IPython configuration,
    you probably want `IPython.start_ipython()` instead.

    Here is a simple example::

        from IPython import embed
        a = 10
        b = 20
        embed(header='First time')
        c = 30
        d = 40
        embed()

    Parameters
    ----------

    header : str
        Optional header string to print at startup.
    compile_flags
        Passed to the `compile_flags` parameter of :py:meth:`terminal.embed.InteractiveShellEmbed.mainloop()`,
        which is called when the :class:`terminal.embed.InteractiveShellEmbed` instance is called.
    **kwargs : various, optional
        Any other kwargs will be passed to the :class:`terminal.embed.InteractiveShellEmbed` constructor.
        Full customization can be done by passing a traitlets :class:`Config` in as the
        `config` argument (see :ref:`configure_start_ipython` and :ref:`terminal_options`).
    """
    ...




from ..classes.graph import Graph

__all__ = ["is_isolate", "isolates", "number_of_isolates"]

def is_isolate(G: Graph, n) -> bool: ...
def isolates(G: Graph): ...
def number_of_isolates(G: Graph) -> int: ...



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2014, Nicolas P. Rougier. All rights reserved.
# Distributed under the terms of the new BSD License.
# -----------------------------------------------------------------------------
from .element import Element
from .transform import Transform

class Transformable(Element):
    def __init__(self, content=None, parent=None): ...
    @property
    def transform(self): ...



from .._shared.utils import deprecated as deprecated

from ._canny import canny as canny
from ._cascade import Cascade as Cascade
from ._daisy import daisy as daisy
from ._hog import hog as hog
from .texture import (
    graycomatrix as graycomatrix,
    graycoprops as graycoprops,
    local_binary_pattern as local_binary_pattern,
    multiblock_lbp as multiblock_lbp,
    draw_multiblock_lbp as draw_multiblock_lbp,
)

from .peak import peak_local_max as peak_local_max
from .corner import (
    corner_kitchen_rosenfeld as corner_kitchen_rosenfeld,
    corner_harris as corner_harris,
    corner_shi_tomasi as corner_shi_tomasi,
    corner_foerstner as corner_foerstner,
    corner_subpix as corner_subpix,
    corner_peaks as corner_peaks,
    corner_fast as corner_fast,
    structure_tensor as structure_tensor,
    structure_tensor_eigenvalues as structure_tensor_eigenvalues,
    structure_tensor_eigvals as structure_tensor_eigvals,
    hessian_matrix as hessian_matrix,
    hessian_matrix_eigvals as hessian_matrix_eigvals,
    hessian_matrix_det as hessian_matrix_det,
    corner_moravec as corner_moravec,
    corner_orientations as corner_orientations,
    shape_index as shape_index,
)
from .template import match_template as match_template
from .brief import BRIEF as BRIEF
from .censure import CENSURE as CENSURE
from .orb import ORB as ORB
from .sift import SIFT as SIFT
from .match import match_descriptors as match_descriptors
from .util import plot_matches as plot_matches
from .blob import blob_dog as blob_dog, blob_log as blob_log, blob_doh as blob_doh
from .haar import (
    haar_like_feature as haar_like_feature,
    haar_like_feature_coord as haar_like_feature_coord,
    draw_haar_like_feature as draw_haar_like_feature,
)
from ._basic_features import multiscale_basic_features as multiscale_basic_features

@deprecated(alt_func="skimage.feature.graycomatrix", removed_version="1.0")
def greycomatrix(
    image, distances, angles, levels=None, symmetric=False, normed=False
): ...
@deprecated(alt_func="skimage.feature.graycoprops", removed_version="1.0")
def greycoprops(P, prop="contrast"): ...

__all__ = [
    "canny",
    "Cascade",
    "daisy",
    "hog",
    "graycomatrix",
    "graycoprops",
    "greycomatrix",
    "greycoprops",
    "local_binary_pattern",
    "multiblock_lbp",
    "draw_multiblock_lbp",
    "peak_local_max",
    "structure_tensor",
    "structure_tensor_eigenvalues",
    "structure_tensor_eigvals",
    "hessian_matrix",
    "hessian_matrix_det",
    "hessian_matrix_eigvals",
    "shape_index",
    "corner_kitchen_rosenfeld",
    "corner_harris",
    "corner_shi_tomasi",
    "corner_foerstner",
    "corner_subpix",
    "corner_peaks",
    "corner_moravec",
    "corner_fast",
    "corner_orientations",
    "match_template",
    "BRIEF",
    "CENSURE",
    "ORB",
    "SIFT",
    "match_descriptors",
    "plot_matches",
    "blob_dog",
    "blob_doh",
    "blob_log",
    "haar_like_feature",
    "haar_like_feature_coord",
    "draw_haar_like_feature",
    "multiscale_basic_features",
]



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.
# -----------------------------------------------------------------------------

from .path_collection import PathCollection as PathCollection  # noqa
from .point_collection import PointCollection as PointCollection  # noqa
from .polygon_collection import PolygonCollection as PolygonCollection  # noqa
from .segment_collection import SegmentCollection as SegmentCollection  # noqa
from .triangle_collection import TriangleCollection as TriangleCollection  # noqa



from typing import ClassVar, Literal, TypeVar
from numpy.random import RandomState
from numpy import ndarray
from ..utils._param_validation import (
    Hidden as Hidden,
    Interval as Interval,
    StrOptions as StrOptions,
    Options as Options,
)
from numbers import Integral as Integral
from pandas.core.series import Series
from ..base import BaseEstimator, TransformerMixin
from scipy.sparse import spmatrix
from ..cluster import KMeans as KMeans
from .._typing import ArrayLike, Int, Float, MatrixLike
from ..utils.validation import (
    check_array as check_array,
    check_is_fitted as check_is_fitted,
    check_random_state as check_random_state,
)
from . import OneHotEncoder as OneHotEncoder

KBinsDiscretizer_Self = TypeVar("KBinsDiscretizer_Self", bound="KBinsDiscretizer")

# Author: Henry Lin <hlin117@gmail.com>
#         Tom Dupré la Tour

# License: BSD


import numpy as np
import warnings


class KBinsDiscretizer(TransformerMixin, BaseEstimator):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    n_bins_: ndarray = ...
    bin_edges_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        n_bins: ArrayLike | Int = 5,
        *,
        encode: Literal["onehot", "onehot-dense", "ordinal", "onehot"] = "onehot",
        strategy: Literal["uniform", "quantile", "kmeans", "quantile"] = "quantile",
        dtype: None | Float = None,
        subsample: int | None | Literal["warn", "warn"] = "warn",
        random_state: RandomState | None | Int = None,
    ) -> None:
        ...

    def fit(
        self: KBinsDiscretizer_Self, X: MatrixLike, y: Series | None | ndarray = None
    ) -> KBinsDiscretizer_Self:
        ...

    def transform(self, X: MatrixLike) -> ndarray | spmatrix:
        ...

    def inverse_transform(self, Xt: MatrixLike) -> ndarray:
        ...

    def get_feature_names_out(self, input_features: None | ArrayLike = None) -> ndarray:
        ...



# Python: 3.8.2 (tags/v3.8.2:7b3ab59, Feb 25 2020, 23:03:10) [MSC v.1916 64 bit (AMD64)]
# Library: scipy, version: 1.6.2
# Module: scipy.fftpack.convolve, version: unspecified
import typing
import builtins as _mod_builtins

__all__: list
__doc__: typing.Any
__file__: str
__name__: str
__package__: str
def __pyx_unpickle_Enum() -> typing.Any:
    ...

__test__: dict
def convolve(x, omega, swap_real_imag=..., overwrite_x=...) -> typing.Any:
    "y = convolve(x,omega,[swap_real_imag,overwrite_x])\n\n    Wrapper for ``convolve``.\n\n    Parameters\n    ----------\n    x : input rank-1 array('d') with bounds (n)\n    omega : input rank-1 array('d') with bounds (n)\n\n    Other Parameters\n    ----------------\n    overwrite_x : input int, optional\n        Default: 0\n    swap_real_imag : input int, optional\n         Default: 0\n\n    Returns\n    -------\n    y : rank-1 array('d') with bounds (n) and x storage\n    "
    ...

def convolve_z(x, omega_real, omega_imag, overwrite_x=...) -> typing.Any:
    "y = convolve_z(x,omega_real,omega_imag,[overwrite_x])\n\n    Wrapper for ``convolve_z``.\n\n    Parameters\n    ----------\n    x : input rank-1 array('d') with bounds (n)\n    omega_real : input rank-1 array('d') with bounds (n)\n    omega_imag : input rank-1 array('d') with bounds (n)\n\n    Other Parameters\n    ----------------\n    overwrite_x : input int, optional\n        Default: 0\n\n    Returns\n    -------\n    y : rank-1 array('d') with bounds (n) and x storage\n    "
    ...

def destroy_convolve_cache() -> typing.Any:
    ...

def init_convolution_kernel(n, kernel_func, d=..., zero_nyquist=..., kernel_func_extra_args=...) -> typing.Any:
    "omega = init_convolution_kernel(n,kernel_func,[d,zero_nyquist,kernel_func_extra_args])\n\n    Wrapper for ``init_convolution_kernel``.\n\n    Parameters\n    ----------\n    n : input int\n    kernel_func : call-back function\n\n    Other Parameters\n    ----------------\n    d : input int, optional\n        Default: 0\n    kernel_func_extra_args : input tuple, optional\n        Default: ()\n    zero_nyquist : input int, optional\n        Default: d%2\n\n    Returns\n    -------\n    omega : rank-1 array('d') with bounds (n)\n\n    Notes\n    -----\n    Call-back functions::\n\n      def kernel_func(k): return kernel_func\n      Required arguments:\n        k : input int\n      Return objects:\n        kernel_func : float\n    "
    ...

def r2r_fftpack(a, axes, real2hermitian, forward, inorm=..., out=..., nthreads=...) -> typing.Any:
    "r2r_fftpack(a: array, axes: object, real2hermitian: bool, forward: bool, inorm: int = 0, out: object = None, nthreads: int = 1) -> array\n\nPerforms a real-valued FFT using the FFTPACK storage scheme.\n\nParameters\n----------\na : numpy.ndarray (any real type)\n    The input data\naxes : list of integers\n    The axes along which the FFT is carried out.\n    If not set, all axes will be transformed.\nreal2hermitian : bool\n    if True, the input is purely real and the output will have Hermitian\n    symmetry and be stored in FFTPACK's halfcomplex ordering, otherwise the\n    opposite.\nforward : bool\n    If `True`, a negative sign is used in the exponent, else a positive one.\ninorm : int\n    Normalization type\n      0 : no normalization\n      1 : divide by sqrt(N)\n      2 : divide by N\n    where N is the length of `axis`.\nout : numpy.ndarray (same shape and data type as `a`)\n    May be identical to `a`, but if it isn't, it must not overlap with `a`.\n    If None, a new array is allocated to store the output.\nnthreads : int\n    Number of threads to use. If 0, use the system default (typically governed\n    by the `OMP_NUM_THREADS` environment variable).\n\nReturns\n-------\nnumpy.ndarray (same shape and data type as `a`)\n    The transformed data. The shape is identical to that of the input array.\n\n"
    ...

def __getattr__(name) -> typing.Any:
    ...




from datetime import date
from typing import Any, Dict, Optional

TIME_STRINGS: Dict[str, str]
TIMESINCE_CHUNKS: Any

def timesince(
    d: date,
    now: Optional[date] = ...,
    reversed: bool = ...,
    time_strings: Optional[Dict[str, str]] = ...,
) -> str: ...
def timeuntil(
    d: date, now: Optional[date] = ..., time_strings: Optional[Dict[str, str]] = ...
) -> str: ...



from typing import Mapping
from collections import defaultdict

from ...classes.graph import Graph

__all__ = [
    "dfs_edges",
    "dfs_tree",
    "dfs_predecessors",
    "dfs_successors",
    "dfs_preorder_nodes",
    "dfs_postorder_nodes",
    "dfs_labeled_edges",
]

def dfs_edges(G: Graph, source=None, depth_limit=None): ...
def dfs_tree(G: Graph, source=None, depth_limit=None): ...
def dfs_predecessors(G: Graph, source=None, depth_limit=None) -> Mapping: ...
def dfs_successors(G: Graph, source=None, depth_limit=None) -> Mapping: ...
def dfs_postorder_nodes(G: Graph, source=None, depth_limit=None): ...
def dfs_preorder_nodes(G: Graph, source=None, depth_limit=None): ...
def dfs_labeled_edges(G: Graph, source=None, depth_limit=None): ...



from ...classes.graph import Graph
from ...utils.decorators import not_implemented_for

__all__ = [
    "number_weakly_connected_components",
    "weakly_connected_components",
    "is_weakly_connected",
]

def weakly_connected_components(G: Graph): ...
def number_weakly_connected_components(G: Graph): ...
def is_weakly_connected(G: Graph) -> bool: ...



from typing import Any, List, Optional, Sequence

from django.apps.config import AppConfig
from django.core.checks.messages import CheckMessage

def check_user_model(
    app_configs: Optional[Sequence[AppConfig]] = ..., **kwargs: Any
) -> List[CheckMessage]: ...
def check_models_permissions(
    app_configs: Optional[Sequence[AppConfig]] = ..., **kwargs: Any
) -> List[Any]: ...



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.
# -----------------------------------------------------------------------------

import os
import sys
import re
import json
import weakref
from distutils.version import LooseVersion

import numpy as np

from . import gl
from ..util import logger

# TODO: expose these via an extension space in .gl?
_internalformats: list = ...
_internalformats = ...

# Value to mark a glir object that was just deleted. So we can safely
# ignore it (and not raise an error that the object could not be found).
# This can happen e.g. if A is created, A is bound to B and then A gets
# deleted. The commands may get executed in order: A gets created, A
# gets deleted, A gets bound to B.
JUST_DELETED: str = ...

def as_enum(enum): ...

class _GlirQueueShare(object):
    def __init__(self, queue): ...
    def command(self, *args): ...
    def set_verbose(self, verbose): ...
    def show(self, filter=None): ...
    def clear(self): ...
    def flush(self, parser): ...
    def _filter(self, commands, parser): ...

class GlirQueue(object):
    def __init__(self): ...
    def command(self, *args): ...
    def set_verbose(self, verbose): ...
    def clear(self): ...
    def associate(self, queue): ...
    def flush(self, parser): ...

def _convert_es2_shader(shader): ...
def _convert_desktop_shader(shader): ...
def convert_shader(backend_type, shader): ...
def as_es2_command(command): ...

class BaseGlirParser(object):
    def __init__(self): ...
    def is_remote(self): ...
    @property
    def shader_compatibility(self): ...
    def parse(self, commands): ...

class GlirParser(BaseGlirParser):
    def __init__(self): ...
    @property
    def shader_compatibility(self): ...
    def is_remote(self): ...
    def _parse(self, command): ...
    def parse(self, commands): ...
    def get_object(self, id_): ...
    def _gl_initialize(self): ...

def glir_logger(parser_cls, file_or_filename): ...

# GLIR objects

class GlirObject(object):
    def __init__(self, parser, id_): ...
    @property
    def handle(self): ...
    @property
    def id(self): ...
    def __repr__(self): ...

class GlirShader(GlirObject):
    _target: None = ...

    def create(self): ...
    def set_data(self, offset, code): ...
    def delete(self): ...
    def _get_error(self, code, errors, indentation=0): ...
    def _parse_error(self, error): ...

class GlirVertexShader(GlirShader):
    _target = ...

class GlirFragmentShader(GlirShader):
    _target = ...

class GlirGeometryShader(GlirShader):
    # _target assignment must be delayed because GL_GEOMETRY_SHADER does not
    # exist until the user calls use_gl('gl+')
    _target: None = ...

    def __init__(self, *args, **kwargs): ...

class GlirProgram(GlirObject):

    UTYPEMAP: dict = ...

    ATYPEMAP: dict = ...

    ATYPEINFO: dict = ...

    def create(self): ...
    def delete(self): ...
    def activate(self): ...
    def deactivate(self): ...
    def set_shaders(self, vert, frag): ...
    def attach(self, id_): ...
    def link_program(self): ...
    def _get_active_attributes_and_uniforms(self): ...
    def set_texture(self, name, value): ...
    def set_uniform(self, name, type_, value): ...
    def set_attribute(self, name, type_, value): ...
    def _pre_draw(self): ...
    def _validate(self): ...
    def _post_draw(self): ...
    def draw(self, mode, selection): ...

class GlirBuffer(GlirObject):
    _target: None = ...
    _usage = ...  # STATIC_DRAW, STREAM_DRAW or DYNAMIC_DRAW

    def create(self): ...
    def delete(self): ...
    def activate(self): ...
    def deactivate(self): ...
    def set_size(self, nbytes): ...
    def set_data(self, offset, data): ...

class GlirVertexBuffer(GlirBuffer):
    _target = ...

class GlirIndexBuffer(GlirBuffer):
    _target = ...

class GlirTexture(GlirObject):
    _target: None = ...

    _types: dict = ...

    def create(self): ...
    def delete(self): ...
    def activate(self): ...
    def deactivate(self): ...

    # Taken from pygly
    def _get_alignment(self, width): ...
    def set_wrapping(self, wrapping): ...
    def set_interpolation(self, min, mag): ...

# these should be auto generated in _constants.py. But that doesn't seem
# to be happening. TODO - figure out why the C parser in (createglapi.py)
# is not extracting these constanst out.
# found the constant value at:
# http://docs.factorcode.org/content/word-GL_TEXTURE_1D,opengl.gl.html
# http://docs.factorcode.org/content/word-GL_SAMPLER_1D%2Copengl.gl.html
GL_SAMPLER_1D = ...
GL_TEXTURE_1D = ...

class GlirTexture1D(GlirTexture):
    _target = ...

    def set_size(self, shape, format, internalformat): ...
    def set_data(self, offset, data): ...

class GlirTexture2D(GlirTexture):
    _target = ...

    def set_size(self, shape, format, internalformat): ...
    def set_data(self, offset, data): ...

GL_SAMPLER_3D = ...
GL_TEXTURE_3D = ...

USE_TEX_3D: bool = ...

def _check_pyopengl_3D(): ...
def glTexImage3D(target, level, internalformat, format, type, pixels): ...
def glTexImage1D(target, level, internalformat, format, type, pixels): ...
def glTexSubImage1D(target, level, xoffset, format, type, pixels): ...
def glTexSubImage3D(target, level, xoffset, yoffset, zoffset, format, type, pixels): ...

class GlirTexture3D(GlirTexture):
    _target = ...

    def set_size(self, shape, format, internalformat): ...
    def set_data(self, offset, data): ...

class GlirTextureCube(GlirTexture):
    _target = ...
    _cube_targets: list = ...

    def set_size(self, shape, format, internalformat): ...
    def set_data(self, offset, data): ...

class GlirRenderBuffer(GlirObject):
    def create(self): ...
    def delete(self): ...
    def activate(self): ...
    def deactivate(self): ...
    def set_size(self, shape, format): ...

class GlirFrameBuffer(GlirObject):

    # todo: on ES 2.0 -> gl.gl_RGBA4
    _formats: dict = ...

    def create(self): ...
    def delete(self): ...
    def set_framebuffer(self, yes): ...
    def activate(self): ...
    def deactivate(self): ...
    def attach(self, attachment, buffer_id): ...
    def _validate(self): ...



from numpy.typing import ArrayLike
from typing import Any
import math
from warnings import warn

import numpy as np
from numpy.linalg import inv
from scipy import optimize, spatial

def _check_data_dim(data, dim): ...
def _check_data_atleast_2D(data): ...

class BaseModel(object):
    def __init__(self): ...

class LineModelND(BaseModel):
    def estimate(self, data) -> bool: ...
    def residuals(self, data, params=None): ...
    def predict(self, x, axis: int = 0, params=None): ...
    def predict_x(self, y: ArrayLike, params=None) -> ArrayLike: ...
    def predict_y(self, x: ArrayLike, params=None) -> ArrayLike: ...

class CircleModel(BaseModel):
    def estimate(self, data) -> bool: ...
    def residuals(self, data): ...
    def predict_xy(self, t: ArrayLike, params=None): ...

class EllipseModel(BaseModel):
    def estimate(self, data) -> bool: ...
    def residuals(self, data): ...
    def predict_xy(self, t: ArrayLike, params=None): ...

def _dynamic_max_trials(n_inliers, n_samples, min_samples, probability): ...
def ransac(
    data,
    model_class: Any,
    min_samples,
    residual_threshold,
    is_data_valid=None,
    is_model_valid=None,
    max_trials: int = 100,
    stop_sample_num: int = ...,
    stop_residuals_sum: float = 0,
    stop_probability=1,
    random_state=None,
    initial_inliers=None,
): ...



from vispy.color import Color

# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.
# -----------------------------------------------------------------------------
# Author: Siddharth Bhat
# -----------------------------------------------------------------------------

import numpy as np

from . import Visual
from ..color import Color

_VERTEX_SHADER: str = ...

_FRAGMENT_SHADER: str = ...  # noqa

class _BorderVisual(Visual):

    _shaders: dict = ...

    def __init__(
        self,
        pos: tuple[float, float],
        halfdim: tuple[float, float],
        border_width: float = 1.0,
        border_color: str | Color | None = None,
        **kwargs,
    ): ...
    @staticmethod
    def _prepare_transforms(view): ...
    @property
    def visual_border_width(self): ...
    def _update(self): ...
    def _prepare_draw(self, view=None): ...
    @property
    def border_width(self): ...
    @border_width.setter
    def border_width(self, border_width): ...
    @property
    def border_color(self): ...
    @border_color.setter
    def border_color(self, border_color): ...
    @property
    def pos(self): ...
    @pos.setter
    def pos(self, pos): ...
    @property
    def halfdim(self): ...
    @halfdim.setter
    def halfdim(self, halfdim): ...



from numpy.typing import ArrayLike

# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.
# -----------------------------------------------------------------------------

import numpy as np

from .triangulation import Triangulation

class PolygonData(object):
    def __init__(
        self,
        vertices: ArrayLike | None = None,
        edges: ArrayLike | None = None,
        faces: ArrayLike | None = None,
    ): ...
    @property
    def faces(self): ...
    @faces.setter
    def faces(self, f): ...
    @property
    def vertices(self): ...
    @vertices.setter
    def vertices(self, v): ...
    @property
    def edges(self): ...
    @edges.setter
    def edges(self, e): ...
    @property
    def convex_hull(self): ...
    def triangulate(self): ...
    def add_vertex(self, vertex: ArrayLike): ...



from numpy.typing import ArrayLike, NDArray

# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

import numpy as np

def stft(
    x: ArrayLike,
    n_fft: int = 1024,
    step: None | int = 512,
    fs: float = ...,
    window: str | None = "hann",
) -> NDArray: ...
def fft_freqs(n_fft: int, fs: float): ...



from typing import Any, Optional

def kml(
    request: Any,
    label: Any,
    model: Any,
    field_name: Optional[Any] = ...,
    compress: bool = ...,
    using: Any = ...,
) -> Any: ...
def kmz(
    request: Any,
    label: Any,
    model: Any,
    field_name: Optional[Any] = ...,
    using: Any = ...,
) -> Any: ...



from .backend_bases import RendererBase
from .transforms import Transform
from typing import Sequence
from ._typing import *
from .path import Path
from .patches import Patch
from .axis import Axis, XAxis, YAxis
from .axes import Axes

from collections import OrderedDict
from functools import partial

from collections.abc import MutableMapping
from .artist import allow_rasterization

class Spine(Patch):
    def __str__(self) -> str: ...
    def __init__(self, axes: Axes, spine_type: str, path: Path, **kwargs) -> None: ...
    def set_patch_arc(
        self, center: Sequence[float], radius: float, theta1: float, theta2: float
    ) -> None: ...
    def set_patch_circle(self, center: Sequence[float], radius: float) -> None: ...
    def set_patch_line(self): ...
    def get_patch_transform(self): ...
    def get_window_extent(self, renderer: RendererBase = ...): ...
    def get_path(self): ...
    def register_axis(self, axis: Axis) -> None: ...
    def clear(self) -> None: ...
    @allow_rasterization
    def draw(self, renderer: RendererBase): ...
    def set_position(self, position: tuple[str, float]) -> None: ...
    def get_position(self) -> tuple[str, float]: ...
    def get_spine_transform(self) -> Transform: ...
    def set_bounds(self, low: float | None = ..., high: float | None = ...) -> None: ...
    def get_bounds(self): ...
    @classmethod
    def linear_spine(cls, axes, spine_type, **kwargs) -> Spine: ...
    @classmethod
    def arc_spine(
        cls,
        axes,
        spine_type,
        center: Sequence[float],
        radius: float,
        theta1: float,
        theta2: float,
        **kwargs
    ) -> Spine: ...
    @classmethod
    def circular_spine(cls, axes, center, radius, **kwargs): ...
    def set_color(self, c: Color) -> None: ...

class SpinesProxy:
    def __init__(self, spine_dict: dict[str, Spine]) -> None: ...
    def __getattr__(self, name: str) -> partial: ...
    def __dir__(self): ...

class Spines(MutableMapping):
    def __init__(self, **kwargs) -> None: ...
    @classmethod
    def from_dict(cls, d: OrderedDict | dict[str, Spine]) -> "Spines": ...
    def __getstate__(self): ...
    def __setstate__(self, state): ...
    def __getattr__(self, name: str) -> Spine: ...
    def __getitem__(self, key: list[str] | slice | str) -> Spine: ...
    def __setitem__(self, key: str, value: Spine) -> None: ...
    def __delitem__(self, key): ...
    def __iter__(self): ...
    def __len__(self) -> int: ...



"""
This type stub file was generated by pyright.
"""

import ast
from typing import Callable, Dict, Union
from prompt_toolkit.filters import Condition
from prompt_toolkit.layout.layout import FocusableElement
from IPython.utils.decorators import undoc

"""
Filters restricting scope of IPython Terminal shortcuts.
"""
@undoc
@Condition
def cursor_in_leading_ws():
    ...

def has_focus(value: FocusableElement): # -> Condition:
    """Wrapper around has_focus adding a nice `__name__` to tester function"""
    ...

@undoc
@Condition
def has_line_below() -> bool:
    ...

@undoc
@Condition
def is_cursor_at_the_end_of_line() -> bool:
    ...

@undoc
@Condition
def has_line_above() -> bool:
    ...

@Condition
def ebivim():
    ...

@Condition
def supports_suspend():
    ...

@Condition
def auto_match():
    ...

def all_quotes_paired(quote, buf):
    ...

_preceding_text_cache: Dict[Union[str, Callable], Condition] = ...
_following_text_cache: Dict[Union[str, Callable], Condition] = ...
def preceding_text(pattern: Union[str, Callable]): # -> Condition:
    ...

def following_text(pattern): # -> Condition:
    ...

@Condition
def not_inside_unclosed_string():
    ...

@Condition
def navigable_suggestions():
    ...

@Condition
def readline_like_completions():
    ...

@Condition
def is_windows_os():
    ...

default_buffer_focused = ...
KEYBINDING_FILTERS = ...
def eval_node(node: Union[ast.AST, None]): # -> None:
    ...

def filter_from_string(code: str): # -> None:
    ...

__all__ = ["KEYBINDING_FILTERS", "filter_from_string"]
